/*
 * MODULE:  ibase.h
 * DESCRIPTION: OSRI entrypoints and defines
 *
 * The contents of this file are subject to the Interbase Public
 * License Version 1.0 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy
 * of the License at http://www.Inprise.com/IPL.html
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code was created by Inprise Corporation
 * and its predecessors. Portions created by Inprise Corporation are
 * Copyright (C) Inprise Corporation.
 *
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 * Added TCP_NO_DELAY option for superserver on Linux
 * FSG 16.03.2001
 * 2001.07.28: John Bellardo:  Added blr_skip
 * 2001.09.18: Ann Harrison:   New info codes
 * 17-Oct-2001 Mike Nordell: CPU affinity
 * 2001-04-16 Paul Beach: ISC_TIME_SECONDS_PRECISION_SCALE modified for HP10
 * Compiler Compatibility
 * 2002.02.15 Sean Leyne - Code Cleanup, removed obsolete ports:
 *                          - EPSON, XENIX, MAC (MAC_AUX), Cray and OS/2
 * 2002.10.29 Nickolay Samofatov: Added support for savepoints
 *
 * 2002.10.29 Sean Leyne - Removed support for obsolete IPX/SPX Protocol
 *
 * 2008.12.14 Takuya Kurosawa - Rewrite for D programming language 2.0. And added Japanese comment.
 */
module org.firebirdsql.ibase;

immutable int FB_API_VER = 20;

immutable int ISC_TRUE = 1;
immutable int ISC_FALSE = 0;

/// Status vector
alias ptrdiff_t ISC_STATUS;
immutable int ISC_STATUS_LENGTH = 20;
alias ISC_STATUS[ISC_STATUS_LENGTH] ISC_STATUS_ARRAY;

/// 日時情報
alias int  ISC_DATE;
alias uint ISC_TIME;
struct ISC_TIMESTAMP
{
	ISC_DATE timestamp_date;
	ISC_TIME timestamp_time;
}

/*******************************************************************/
/* Blob Id support                                                 */
/*******************************************************************/
struct GDS_QUAD_t
{
	int gds_quad_high;
	uint gds_quad_low;

	alias gds_quad_high isc_quad_high;
	alias gds_quad_low isc_quad_low;
}

alias GDS_QUAD_t GDS_QUAD;
alias GDS_QUAD_t ISC_QUAD;

/********************************/
/* InterBase Handle Definitions */
/********************************/
/// API handles
typedef uint FB_API_HANDLE;

typedef FB_API_HANDLE isc_att_handle;
typedef FB_API_HANDLE isc_blob_handle;
typedef FB_API_HANDLE isc_db_handle;
typedef FB_API_HANDLE isc_req_handle;
typedef FB_API_HANDLE isc_stmt_handle;
typedef FB_API_HANDLE isc_svc_handle;
typedef FB_API_HANDLE isc_tr_handle;
typedef int  isc_resv_handle;

typedef void function()         isc_callback;
typedef void function(void*, short, char*)   ISC_PRINT_CALLBACK;
typedef void function(void*, char*)      ISC_VERSION_CALLBACK;
typedef void function(void*, ushort, ubyte*) ISC_EVENT_CALLBACK;

/*******************************************************************/
/* Blob id structure                                               */
/*******************************************************************/
struct ISC_ARRAY_BOUND
{
	short array_bound_lower;
	short array_bound_upper;
}

struct ISC_ARRAY_DESC
{
	ubyte   array_desc_dtype;
	char   array_desc_scale;
	ushort    array_desc_length;
	char[32]  array_desc_field_name;
	char[32]  array_desc_relation_name;
	short    array_desc_dimensions;
	short    array_desc_flags;
	ISC_ARRAY_BOUND[16] array_desc_bounds;
}

/++
 + Blobデータの情報を表す構造体
 +/
struct ISC_BLOB_DESC
{
	/++
	 + 0未満 ユーザー定義
	 + 0  不明タイプ
	 + 1  テキスト
	 + 2  バイナリー
	 + 3  アクセスコントロールリスト
	 + 4  (予約)
	 + 5  エンコードされたメタデータ
	 + 6  異常終了した多重データ・ベース処理の記述
	 +/
	short blob_desc_subtype;

	/// 文字コード
	short blob_desc_charset;

	/// セグメントサイズ
	short blob_desc_segment_size;

	/// カラム名 (文字列は0終端)
	ubyte[32] blob_desc_field_name;

	/// テーブル名 (文字列は0終端)
	ubyte[32] blob_desc_relation_name;
}

/***************************/
/* Blob control structure  */
/***************************/
struct isc_blob_ctl
{
	ISC_STATUS function() ctl_source;    /* Source filter */
	isc_blob_ctl*   ctl_source_handle;  /* Argument to pass to source filter */
	short     ctl_to_sub_type;  /* Target type */
	short     ctl_from_sub_type;  /* Source type */
	ushort     ctl_buffer_length;  /* Length of buffer */
	ushort     ctl_segment_length;  /* Length of current segment */
	ushort     ctl_bpb_length;   /* Length of blob parameter  block */
	/* Internally, this is const UCHAR*, but this public struct probably can't change. */
	char*    ctl_bpb;    /* Address of blob parameter block */
	ubyte*    ctl_buffer;    /* Address of segment buffer */
	int    ctl_max_segment;  /* Length of longest segment */
	int    ctl_number_segments; /* Total number of segments */
	int    ctl_total_length;  /* Total length of blob */
	ISC_STATUS*    ctl_status;    /* Address of status vector */
	int[8]     ctl_data;    /* Application specific data */
}
alias isc_blob_ctl* ISC_BLOB_CTL;

/***************************/
/* Blob stream definitions */
/***************************/

struct bstream
{
	isc_blob_handle bstr_blob;  /* Blob handle */
	char*  bstr_buffer; /* Address of buffer */
	char*  bstr_ptr;  /* Next character */
	short   bstr_length; /* Length of buffer */
	short   bstr_cnt;  /* Characters in buffer */
	byte   bstr_mode;  /* (mode) ? OUTPUT : INPUT */
}
alias bstream BSTREAM;

/* Three ugly macros, one even using octal radix... sigh... */
int getb(BSTREAM* p)
{
	return p.bstr_cnt >= 0 ? *p.bstr_ptr++ & 0xff : BLOB_get(p);
}

int putb(char x, BSTREAM* p)
{
	return (x == '\n' || --p.bstr_cnt == 0) ? BLOB_put(x,p) : cast(int)(*p.bstr_ptr++ = cast(ubyte)x);
}

int putbx(char x, BSTREAM* p)
{
	return --p.bstr_cnt == 0 ? BLOB_put(x,p) : cast(int)(*p.bstr_ptr++ = cast(ubyte)x);
}

/********************************************************************/
/* CVC: Public blob interface definition held in val.h.             */
/* For some unknown reason, it was only documented in langRef       */
/* and being the structure passed by the engine to UDFs it never    */
/* made its way into this public definitions file.                  */
/* Being its original name "blob", I renamed it blobcallback here.  */
/* I did the full definition with the proper parameters instead of  */
/* the weak C declaration with any number and type of parameters.   */
/* Since the first parameter -BLB- is unknown outside the engine,   */
/* it's more accurate to use void* than int* as the blob pointer    */
/********************************************************************/

/* Blob passing structure */

/* This enum applies to parameter "mode" in blob_lseek */
enum blob_lseek_mode
{
	blb_seek_relative = 1,
	blb_seek_from_tail = 2
};

/* This enum applies to the value returned by blob_get_segment */
enum blob_get_result
{
	blb_got_fragment = -1,
	blb_got_eof = 0,
	blb_got_full_segment = 1
};

struct blobcallback
{
    short function(void* hnd, ubyte* buffer, ushort buf_size, ushort* result_len) blob_get_segment;
    void*  blob_handle;
    int blob_number_segments;
    int blob_max_segment;
    int blob_total_length;
    void function(void* hnd, ubyte* buffer, ushort buf_size) blob_put_segment;
    int function(void* hnd, ushort mode, int offset) blob_lseek;
}
alias blobcallback* BLOBCALLBACK;


/********************************************************************/
/* CVC: Public descriptor interface held in dsc2.h.                  */
/* We need it documented to be able to recognize NULL in UDFs.      */
/* Being its original name "dsc", I renamed it paramdsc here.       */
/* Notice that I adjust to the original definition: contrary to     */
/* other cases, the typedef is the same struct not the pointer.     */
/* I included the enumeration of dsc_dtype possible values.         */
/* Ultimately, dsc2.h should be part of the public interface.        */
/********************************************************************/

/* This is the famous internal descriptor that UDFs can use, too. */
struct paramdsc
{
	ubyte dsc_dtype;
	byte  dsc_scale;
	ushort dsc_length;
	short  dsc_sub_type;
	ushort dsc_flags;
	ubyte* dsc_address;
}
alias paramdsc PARAMDSC;

/* This is a helper struct to work with varchars. */
struct paramvary
{
	ushort  vary_length;
	ubyte[1] vary_string;
}
alias paramvary PARAMVARY;

/*
 * The following flags are used in an internal structure dsc (dsc.h) or in the external one paramdsc (ibase.h)
 */

/* values for dsc_flags
 * Note: DSC_null is only reliably set for local variables (blr_variable)
 */
immutable int DSC_null = 1;
immutable int DSC_no_subtype = 2; /* dsc has no sub type specified */
immutable int DSC_nullable = 4; /* not stored. instead, is derived
	          from metadata primarily to flag
	          SQLDA (in DSQL)               */

immutable int dtype_unknown = 0;
immutable int dtype_text = 1;
immutable int dtype_cstring = 2;
immutable int dtype_varying = 3;

immutable int dtype_packed = 6;
immutable int dtype_byte = 7;
immutable int dtype_short = 8;
immutable int dtype_long = 9;
immutable int dtype_quad = 10;
immutable int dtype_real = 11;
immutable int dtype_double = 12;
immutable int dtype_d_float = 13;
immutable int dtype_sql_date = 14;
immutable int dtype_sql_time = 15;
immutable int dtype_timestamp = 16;
immutable int dtype_blob = 17;
immutable int dtype_array = 18;
immutable int dtype_int64 = 19;
immutable int DTYPE_TYPE_MAX = 20;


immutable int ISC_TIME_SECONDS_PRECISION = 10000;
immutable int ISC_TIME_SECONDS_PRECISION_SCALE = -4;


/***************************/
/* Dynamic SQL definitions */
/***************************/


/* Definitions for DSQL free_statement routine */

immutable int DSQL_close = 1;
immutable int DSQL_drop = 2;

/* Declare the extended SQLDA */

struct XSQLVAR
{
	short  sqltype;   /* datatype of field */
	short  sqlscale;   /* scale factor */
	short  sqlsubtype;   /* datatype subtype - currently BLOBs only */

	/// sqldataの長さ
	ushort sqllen;

	/// 実データのポインタ
	ubyte* sqldata;

	/// NULLフラグへのポインタ
	short* sqlind;

	short  sqlname_length;  /* length of sqlname field */
	char[32] sqlname;   /* name of field, name length + space for NULL */
	short  relname_length;  /* length of relation name */
	char[32] relname;   /* field's relation name + space for NULL */
	short  ownname_length;  /* length of owner name */
	char[32] ownname;   /* relation's owner name + space for NULL */
	short  aliasname_length; /* length of alias name */
	char[32] aliasname;   /* relation's alias name + space for NULL */
}

immutable int SQLDA_VERSION1 = 1;

struct XSQLDA
{
	short  ver;  // XSQLDA構造体バージョン 元の識別子はversionだが、D言語予約語の為verに変更
	char[8] sqldaid; /* XSQLDA name field */
	int  sqldabc; /* length in bytes of SQLDA */
	short  sqln;  /* number of fields allocated */
	short  sqld;  /* actual number of fields */
	XSQLVAR[1]  sqlvar;  /* first field address */
}

int XSQLDA_LENGTH(int n)
{
	return cast(int)(XSQLDA.sizeof + (n-1) * XSQLVAR.sizeof);
}

immutable int SQL_TEXT = 452;
immutable int SQL_VARYING = 448;
immutable int SQL_SHORT = 500;
immutable int SQL_LONG = 496;
immutable int SQL_FLOAT = 482;
immutable int SQL_DOUBLE = 480;
immutable int SQL_D_FLOAT = 530;
immutable int SQL_TIMESTAMP = 510;
immutable int SQL_BLOB = 520;
immutable int SQL_ARRAY = 540;
immutable int SQL_QUAD = 550;
immutable int SQL_TYPE_TIME = 560;
immutable int SQL_TYPE_DATE = 570;
immutable int SQL_INT64 = 580;

/* Historical alias for pre v6 code */
immutable int SQL_DATE = SQL_TIMESTAMP;

/***************************/
/* SQL Dialects            */
/***************************/

immutable int SQL_DIALECT_V5 = 1; /* meaning is same as DIALECT_xsqlda */
immutable int SQL_DIALECT_V6_TRANSITION = 2; /* flagging anything that is delimited
	            by double quotes as an error and
	            flagging keyword DATE as an error */
immutable int SQL_DIALECT_V6 = 3; /* supports SQL delimited identifier,
	            SQLDATE/DATE, TIME, TIMESTAMP,
	            CURRENT_DATE, CURRENT_TIME,
	            CURRENT_TIMESTAMP, and 64-bit exact
	            numeric type */
immutable int SQL_DIALECT_CURRENT = SQL_DIALECT_V6; /* latest IB DIALECT */

/++
 + Transaction Existence Blocks (TEB)
 +
 + isc_start_multipleで使用する構造体
 + 接続対象のデータベースとトランザクションのパラメータを保持する
 +/
struct ISC_TEB
{
	/// データベースハンドルのポインタ
	isc_db_handle* db_ptr;

	// tpb_ptrの指すバッファのサイズ
	int tpb_len;

	// isc_tpb_* の設定を指定したバッファ
	byte* tpb_ptr;
}

/***************************/
/* OSRI database functions */
/***************************/

extern (C)
{
/++
 + データベースに接続する
 +
 + プログラムからアクセス可能なデータベースに接続します
 + また、様々なオプションを指定することもできます
 + 例えば、リモートにあるデータベースにユーザーとパスワードを指定し接続したり、
 + キャッシュをいくつ使うかを指定したりできます。
 + これらの設定はDatabase parameter buffer(DPB)によって設定します。
 + DPBはプログラムで直接設定するか、isc_modify_dpbで設定します。
 +
 + Params:
 +  status_vector = エラーステータスを受け取るISC_STATUS配列へのポインタ
 +  db_name_length = databaseに渡す文字列の長さ
 +  db_name = 接続先データベースを文字列で指定する
 +  db_handle = 接続したデータベースのハンドルを受け取るポインタ
 +  parm_buffer_length = dpbの長さ
 +  parm_buffer = 接続時に使用するDatabaseParameterBufferのポインタ
 +
 + Returns:
 +  status_vector[1]の値を返します
 +  成功した場合は0となります
 +  失敗した場合は非0となります
 +  また、エラーが発生した場合はstatus_vector[0]を1にセットし、status_vector[1]にエラーコードをセットします
 +/
ISC_STATUS isc_attach_database(ISC_STATUS* status_vector, short db_name_length, const(char)* db_name, isc_db_handle* db_handle, short parm_buffer_length, const(ubyte)* parm_buffer);

/++
 + データベースから切断する
 +
 + 接続済みデータベースから切断し、
 + 確保されているリソースの開放を行います
 +
 + Params:
 +  status_vector = エラーステータスを受け取るISC_STATUS配列へのポインタ
 +  db_handle = 接続したデータベースのハンドルのポインタ
 +
 + Returns:
 +  status_vector[1]の値を返します
 +  成功した場合は0となります
 +  失敗した場合は非0となります
 +  また、エラーが発生した場合はstatus_vector[0]を1にセットし、status_vector[1]にエラーコードをセットします
 +/
ISC_STATUS isc_detach_database(ISC_STATUS* status_vector, isc_db_handle* db_handle);

ISC_STATUS isc_array_gen_sdl(ISC_STATUS* a, const(ISC_ARRAY_DESC)* b, short* c, ubyte* d, short* e);

ISC_STATUS isc_array_get_slice(ISC_STATUS* a, isc_db_handle* b, isc_tr_handle* c, ISC_QUAD* d, const(ISC_ARRAY_DESC)* e, void* f, int* g);

ISC_STATUS isc_array_lookup_bounds(ISC_STATUS* a, isc_db_handle* b, isc_tr_handle* c, const(char)* d, const(char)* e, ISC_ARRAY_DESC* f);

ISC_STATUS isc_array_lookup_desc(ISC_STATUS* a, isc_db_handle* b, isc_tr_handle* c, const(char)* d, const(char)* e, ISC_ARRAY_DESC* f);

ISC_STATUS isc_array_set_desc(ISC_STATUS* a, const(char)* b, const(char)* c, const(short)* d, const(short)* e, const(short)* f, ISC_ARRAY_DESC* g);

ISC_STATUS isc_array_put_slice(ISC_STATUS* a, isc_db_handle* b, isc_tr_handle* c, ISC_QUAD* d, const(ISC_ARRAY_DESC)* e, void* f, int* g);

/++
 + ISC_BLOB_DESCのデフォルト値を設定する
 +
 + テーブル名とカラム名を指定してISC_BLOB_DESCのデフォルト値を設定する
 + この設定したISC_BLOB_DESCはisc_blob_gen_bpbの呼び出しで使用する
 +
 + Params:
 +  desc = データが書き込まれる構造体へのポインタ
 +  table_name = テーブル名
 +  column_name = カラム名
 +/
void isc_blob_default_desc(ISC_BLOB_DESC* desc, const(ubyte)* table_name, const(ubyte)* column_name);

/++
 + BPB(Blob parameter buffer)を生成
 +
 + Params:
 +  status_vector = エラーステータスを受け取るISC_STATUS配列へのポインタ
 +  to_desc = 
 +  from_desc = 元のISC_BLOB_DESCのポインタ
 +  bpb_buffer_length = 
 +  bpb_buffer = 
 +  bpb_length = 
 +/
ISC_STATUS isc_blob_gen_bpb(ISC_STATUS* status_vector, const(ISC_BLOB_DESC)* to_desc, const(ISC_BLOB_DESC)* from_desc, ushort bpb_buffer_length, ubyte* bpb_buffer, ushort* bpb_length);

ISC_STATUS isc_blob_info(ISC_STATUS* a, isc_blob_handle* b, short c, const(char)* d, short e, char* f);

ISC_STATUS isc_blob_lookup_desc(ISC_STATUS* a, isc_db_handle* b, isc_tr_handle* c, const(ubyte)* d, const(ubyte)* e, ISC_BLOB_DESC* f, ubyte* g);

ISC_STATUS isc_blob_set_desc(ISC_STATUS* a, const(ubyte)* b, const(ubyte)* c, short d, short e, short f, ISC_BLOB_DESC* g);

ISC_STATUS isc_create_blob(ISC_STATUS* a, isc_db_handle* b, isc_tr_handle* c, isc_blob_handle* d, ISC_QUAD* e);

/++
 + blobデータの作成
 +
 + blobデータを作成し、書き込みアクセスを開く
 + また、転送時のフィルタ指定も行います
 +
 + Params:
 +  status_vector = エラーステータスを受け取るISC_STATUS配列へのポインタ
 +  db_handle = データベース接続ハンドルへのポインタ
 +  trans_handle = トランザクションハンドルへのポインタ
 +  blob_handle = blobハンドルへのポインタ
 +  blob_id = blobのIDへのポインタ(レコードに保存されているIDへのポインタ)
 +  bpb_length = blob parameter bufferの長さ
 +  bpb_address = blob parameter bufferのポインタ
 +
 + Returns:
 +  status_vector[1]の値を返します
 +  成功した場合は0となります
 +  失敗した場合は非0となります
 +  また、エラーが発生した場合はstatus_vector[0]を1にセットし、status_vector[1]にエラーコードをセットします
 +/
ISC_STATUS isc_create_blob2(ISC_STATUS* status_vector, isc_db_handle* db_handle, isc_tr_handle* trans_handle, isc_blob_handle* blob_handle, ISC_QUAD* blob_id, ushort bpb_length, const(ubyte)* bpb_address);

ISC_STATUS isc_open_blob(ISC_STATUS* a, isc_db_handle* b, isc_tr_handle* c, isc_blob_handle* d, ISC_QUAD* e);

/++
 + blobデータを開く
 +
 + 既に存在するblobへのアクセスを開く
 + また、転送時のフィルタ指定も行います
 +
 + Params:
 +  status_vector = エラーステータスを受け取るISC_STATUS配列へのポインタ
 +  db_handle = データベース接続ハンドルへのポインタ
 +  trans_handle = トランザクションハンドルへのポインタ
 +  blob_handle = blobハンドルへのポインタ
 +  blob_id = blobのIDへのポインタ(レコードに保存されているIDへのポインタ)
 +  bpb_length = blob parameter bufferの長さ
 +  bpb_address = blob parameter bufferのポインタ
 +
 + Returns:
 +  status_vector[1]の値を返します
 +  成功した場合は0となります
 +  失敗した場合は非0となります
 +  また、エラーが発生した場合はstatus_vector[0]を1にセットし、status_vector[1]にエラーコードをセットします
 +/
ISC_STATUS isc_open_blob2(ISC_STATUS* status_vector, isc_db_handle* db_handle, isc_tr_handle* trans_handle, isc_blob_handle* blob_handle, ISC_QUAD* blob_id, ushort bpb_length, const(ubyte)* bpb_address);

/++
 + blobデータを読み出す
 +
 + 呼び出し毎に次のデータを順次読み出す
 +
 + Params:
 +  status_vector = エラーステータスを受け取るISC_STATUS配列へのポインタ
 +  blob_handle = blobハンドルへのポインタ
 +  actual_seg_length = 実際にバッファに書き込んだデータのサイズ
 +  seg_buffer_length = 読み出したデータを書き込むバッファのサイズ
 +  seg_buffer = 読み出したデータを書き込むバッファのポインタ
 +
 + Returns:
 +  status_vector[1]の値を返します
 +  成功した場合は0となります
 +  読み出しが間に合って居ない場合はisc_segmentを返します。0が帰った時と同様に、再度isc_get_segmentを呼び出します
 +  最後まで読み出した場合はisc_segstr_eofを返します
 +  失敗した場合は非0となります
 +  また、エラーが発生した場合はstatus_vector[0]を1にセットし、status_vector[1]にエラーコードをセットします
 +/
ISC_STATUS isc_get_segment(ISC_STATUS* status_vector, isc_blob_handle* blob_handle, ushort* actual_seg_length, ushort seg_buffer_length, ubyte* seg_buffer);

/++
 + blobデータを書き込む
 +
 + Params:
 +  status_vector = エラーステータスを受け取るISC_STATUS配列へのポインタ
 +  blob_handle = blobハンドルへのポインタ
 +  seg_buffer_length = 書き込むデータバッファのサイズ
 +  seg_buffer = 書き込むデータバッファのポインタ
 +
 + Returns:
 +  status_vector[1]の値を返します
 +  成功した場合は0となります
 +  失敗した場合は非0となります
 +  また、エラーが発生した場合はstatus_vector[0]を1にセットし、status_vector[1]にエラーコードをセットします
 +/
ISC_STATUS isc_put_segment(ISC_STATUS* status_vector, isc_blob_handle* blob_handle, ushort seg_buffer_length, const(ubyte)* seg_buffer);

ISC_STATUS isc_cancel_blob(ISC_STATUS* a, isc_blob_handle* b);

/++
 + blobデータへのアクセスを閉じる
 +
 + isc_create_blob2, isc_open_blob2で開いたblobデータへのアクセスを閉じる
 +
 + Params:
 +  status_vector = エラーステータスを受け取るISC_STATUS配列へのポインタ
 +  blob_handle = ハンドルへのポインタ
 +
 + Returns:
 +  status_vector[1]の値を返します
 +  成功した場合は0となります
 +  失敗した場合は非0となります
 +  また、エラーが発生した場合はstatus_vector[0]を1にセットし、status_vector[1]にエラーコードをセットします
 +/
ISC_STATUS isc_close_blob(ISC_STATUS* status_vector, isc_blob_handle* blob_handle);

ISC_STATUS isc_commit_retaining(ISC_STATUS* a, isc_tr_handle* b);

ISC_STATUS isc_cancel_events(ISC_STATUS* a, isc_db_handle* b, int* c);

ISC_STATUS isc_create_database(ISC_STATUS* a, short b, const(char)* c, isc_db_handle* d, short e, const(char)* f, short g);

ISC_STATUS isc_database_info(ISC_STATUS* a, isc_db_handle* b, short c, const(char)* d, short e, char* f);

void isc_decode_date(const(ISC_QUAD)* a, void* b);

void isc_decode_sql_date(const(ISC_DATE)* a, void* b);

void isc_decode_sql_time(const(ISC_TIME)* a, void* b);

void isc_decode_timestamp(const(ISC_TIMESTAMP)* a, void* b);

ISC_STATUS isc_drop_database(ISC_STATUS* a, isc_db_handle* b);

/++
 + dynamic SQL(DSQL)に使用するステートメントハンドルを作成
 +
 + DSQLに使用するステートメントハンドルを作成し、ポインタを返します
 + このポインタはisc_dsql_prepare()に渡し、特定のSQL文と結び付けます。
 +
 + DSQLを複数回実行する場合やDSQLが結果を返す場合、
 + isc_dsql_prepare()、isc_dsql_execute()を実行する前に
 + このisc_dsql_allocate_statement()、またはisc_dsql_alloc_statement2()でステートメント領域を確保します。
 +
 + isc_dsql_allocate_statement()とisc_dsql_alloc_statement2()は非常によく似ていますが、
 + isc_dsql_allocate_statement()が確保したステートメントハンドルは、
 + データベースから切断された時に自動的にNULLリセットされません。
 + 自動的にNULLリセットしたい場合にisc_dsql_alloc_statement2()を使用してください。
 +
 + ステートメントを使い終わったら、isc_dsql_free_statementで領域を解放してください。
 + isc_detach_database()で切断した場合でも解放されます。
 + 
 + Params:
 +  status_vector = エラーステータスを受け取るISC_STATUS配列へのポインタ
 +  db_handle = SQLを実行する対象のデータベースのハンドル
 +  stmt_handle = ステートメントハンドルを受け取るポインタ ハンドルはNULLで初期化しておかなければならない
 +
 + Returns:
 +  status_vector[1]の値を返します
 +  成功した場合は0となります
 +  失敗した場合は非0となります
 +  また、エラーが発生した場合はstatus_vector[0]を1にセットし、status_vector[1]にエラーコードをセットします
 +/
ISC_STATUS isc_dsql_allocate_statement(ISC_STATUS* status_vector, isc_db_handle* db_handle, isc_stmt_handle* stmt_handle);

ISC_STATUS isc_dsql_alloc_statement2(ISC_STATUS* a, isc_db_handle* b, isc_stmt_handle* c);

ISC_STATUS isc_dsql_describe(ISC_STATUS* a, isc_stmt_handle* b, ushort c, XSQLDA* d);

/++
 + 予め解析されたDSQLを元に、引数の情報をセットする
 +
 + stmt_handleに指定したステートメントから、引数情報を抜き出し
 + xsqldaで指定した領域に結果を書き込む
 +
 + ステートメントを実行する際、引数があればXSQLDAにデータをセットしなければなりません。
 + アプリケーション実行前に引数が何個あり、それぞれの型が何という情報が分かっていれば
 + isc_dsql_describe_bind()を使わず直接XSQLDAを設定することもできます。
 + しかし、事前にそれが分からない場合、isc_dsql_describe_bind()を使って情報を取得してください。
 +
 + Params:
 +  status_vector = エラーステータスを受け取るISC_STATUS配列へのポインタ
 +  stmt_handle = ステートメントハンドルのポインタ 予めisc_dsql_allocate_statement()またはisc_dsql_alloc_statement2()で確保しておく NULLを指定するとエラーを返す
 +  da_version = xsqldaに指定する構造体のバージョン 1しかないので1を指定する
 +  xsqlda = 入力領域のポインタ ここに結果が書き込まれます
 +
 + Returns:
 +  成功した場合は0、失敗した場合は非0を返す
 +/
ISC_STATUS isc_dsql_describe_bind(ISC_STATUS* status_vector, isc_stmt_handle* stmt_handle, ushort da_version, XSQLDA* xsqlda);

ISC_STATUS isc_dsql_exec_immed2(ISC_STATUS* a, isc_db_handle* b, isc_tr_handle* c, ushort d, const(char)* e, ushort f, XSQLDA* g, XSQLDA* h);

/++
 + 予め準備されたDSQLを実行
 +
 + isc_dsql_execute()は次の2つのステートメントを実行できます。
 + 1. 1つ以上のレコードを返す可能性があるステートメント
 + 2. 2回以上実行する必要があるステートメント
 +
 + ステートメントに引数がある場合、データを設定したXSQLDA構造体を指定する必要があります。
 + このAPIの引数に指定するXSQLDAは、取得したデータを受け取るためのものではありません。
 + SELECTしたデータを取得するにはisc_dsql_fetch()を使用してください。
 + isc_dsql_fetch()呼ぶごとに1レコードのデータを取得することができます。
 +
 + EXECUTE PROCEDUREのように、引数と戻り値があるステートメントを繰り返し実行する場合、
 + isc_dsql_execute2()を使用してください。
 +
 + ステートメントを1度だけ実行し、かつ戻り値がない場合は、
 + isc_dsql_execute_immediate()を使用してください。
 + 引数と戻り値があるステートメントを1度だけ実行するときはisc_dsql_exec_immed2()を使用してください。
 +
 + CREATE DTABASEとSET TRANSACTIONはisc_dsql_execute()またはisc_dsqlexecute2()では実行できません。
 + これらを実行するにはisc_dsql_execute_immediate()を使用してください。
 +
 + Params:
 +  status_vector = エラーステータスを受け取るISC_STATUS配列へのポインタ
 +  trans_handle = トランザクションハンドルのポインタ 予めisc_start_transaction()で設定しておかなければならない NULLを指定するとエラーを返す。
 +  stmt_handle = ステートメントハンドルのポインタ 予めisc_dsql_allocate_statement()またはisc_dsql_alloc_statement2()で確保しておく NULLを指定するとエラーを返す
 +  da_version = xsqldaに指定する構造体のバージョン 1しかないので1を指定する
 +  xsqlda = プレースホルダに渡すデータがセットされたXSQLDA構造体のポインタ
 +
 + Returns:
 +  status_vector[1]の値を返します
 +  成功した場合は0となります
 +  失敗した場合は非0となります
 +  また、エラーが発生した場合はstatus_vector[0]を1にセットし、status_vector[1]にエラーコードをセットします
 +/
ISC_STATUS isc_dsql_execute(ISC_STATUS* status_vector, isc_tr_handle* trans_handle, isc_stmt_handle* stmt_handle, ushort da_version, XSQLDA* xsqlda);

ISC_STATUS isc_dsql_execute2(ISC_STATUS* a, isc_tr_handle* b, isc_stmt_handle* c, ushort d, XSQLDA* e, XSQLDA* f);

/++
 + SQLを実行する
 +
 + ステートメント無しでSQLの実行を行う
 + データ取得が必要なSQLには使用できない
 + また、CREATE DATABSE時にはdb_handleとtrans_handleをNULLで初期化し、
 + そのポインタをセットする
 + データベース作成が成功すると、db_handleに作成されたデータベースに接続したハンドルがセットされる
 +
 + Params:
 +  status_vector = エラーステータスを受け取るISC_STATUS配列へのポインタ
 +  db_handle = データベースハンドルのポインタ
 +  trans_handle = トランザクションハンドルのポインタ
 +  length = statementで指定するSQL文の長さ statementに0終端文字列を渡す場合は0と指定してもよい
 +  statement = 実行するSQLを指定する
 +  dialect = データ型の保持内容等、微妙にSQLの扱いが違うバージョンの指定 詳しい記述が見つからないが1がデフォルトのようなのでそれを指定すればよいだろう
 +  xsqlda = XSQLDAのポインタ とりあえずNULLでよい？
 +
 + Returns:
 +  成功した場合は0、失敗した場合は非0を返す
 +/
ISC_STATUS isc_dsql_execute_immediate(ISC_STATUS* status_vector, isc_db_handle* db_handle, isc_tr_handle* trans_handle, ushort length, const(char)* statement, ushort dialect, XSQLDA* xsqlda);

ISC_STATUS isc_dsql_fetch(ISC_STATUS* a, isc_stmt_handle* b, ushort c, XSQLDA* d);

ISC_STATUS isc_dsql_finish(isc_db_handle* a);

/++
 + ステートメントハンドルに付随するリソースの開放、または参照したカーソルのクローズを行う
 +
 + optionにDSQL_closeを指定した場合
 +  ステートメントに関連するカーソルを閉じる
 + optionにDSQL_dropを指定した場合
 +  ステートメントハンドルとそれに付随するリソースの開放を行う
 +  また、ステートメントに関連するカーソルも閉じる
 + optionに上記以外を指定した場合
 +  この関数はなにもしない
 +
 + Params:
 +  status_vector = エラーステータスを受け取るISC_STATUS配列へのポインタ
 +  stmt_handle = ステートメントハンドルのポインタ
 +  option = DSQL_close or DSQL_drop
 +
 + Returns:
 +  成功した場合は0、失敗した場合は非0を返す
 +/
ISC_STATUS isc_dsql_free_statement(ISC_STATUS* status_vector, isc_stmt_handle* stmt_handle, ushort option);

ISC_STATUS isc_dsql_insert(ISC_STATUS* a, isc_stmt_handle* b, ushort c, XSQLDA* d);

/++
 + 繰り返し実行のためにDSQLを準備する
 +
 + isc_dsql_prepare()は繰り返し実行用に指定されたDSQLステートメントの
 + シンタックスエラーをチェックし、効率的な実行のために解析し、実行準備をします。
 + 全てのSELECT文はisc_dsql_prepare()で準備しなければなりません。
 +
 + 1度ステートメントを準備した後は、セッション終了まで複数回DSQLを実行できます。
 + よって、DSQLを繰り返し実行する場合はisc_dsql_execute_immediate()やisc_dsql_exec_immed2()よりも効率的に動作します。
 +
 + もしステートメントに戻り値がない場合は、xsqldaにはNULLを指定します。
 + そうでない場合は、関数実行前に予め領域を確保しておかなければなりません。
 + XSQLDAを確保する場合は、XSQLDA_LENGTHを使用してください。
 + XSQLDA_LENGTHは引数に指定したカラム数が何バイト必要かを計算します。
 +
 + XSQLDAを確保した後は、xsqlda->verにSQLDA_VERSION1を、xsqlda->sqlnにXSQLDA_LENGTHに指定したカラム数を設定します。
 + そして、isc_dsql_prepare()を使用すると、上記以外のXSQLDAとXSQLVARのフィールド、
 + つまり、データの型や長さ、SELECT文のカラムの名称が設定されます。
 + 
 + xsqlda->sqldにはSELECT文が実際に返すカラムの個数が設定されます。
 + もし、xsqlda->sqldがxsqlda->sqlnより大きくなった場合は、十分な領域が確保されていないことになります。
 + よって、XSQLDAを次の手順で拡張しなければなりません。
 +
 + 1. xsqlda->sqldの値を退避しておく
 + 2. 確保済みのXSQLDAを解放
 + 3. 1で退避しておいた値をXSQLDA_LENGTHに使用し、新たに領域を確保
 + 4. xsqlda->sqlnとxsqlda->versionを設定
 + 5. isc_dsql_describe()を使用して他の値を設定
 + 
 + 準備したDSQLが引数を持つ場合、isc_dsql_execute()やisc_dsql_execute2()で実行する前に
 + 引数用のXSQLDAを確保し、適切な値を設定しておく必要があります。
 + 引数が分かっていれば、直接XSQLDAを確保し、値を設定することができます。
 + そうでない場合はisc_dsql_describe_bind()を使用すれば引数の情報を取得し、設定することができます。
 +
 + Params:
 +  status_vector = エラーステータスを受け取るISC_STATUS配列へのポインタ
 +  trans_handle = トランザクションハンドルのポインタ 予めisc_start_transaction()で設定しておかなければならない NULLを指定するとエラーを返す。
 +  stmt_handle = ステートメントハンドルのポインタ 予めisc_dsql_allocate_statement()またはisc_dsql_alloc_statement2()で確保しておく NULLを指定するとエラーを返す
 +  length = statementの文字列長 C言語のNULL終端文字列の場合は0を指定する
 +  statement = SQLの文字列
 +  dialect = データ型の保持内容等、微妙にSQLの扱いが違うバージョンの指定 詳しい記述が見つからないが1がデフォルトのようなのでそれを指定すればよいだろう
 +  xsqlda = 結果情報用に予め確保された領域へのポインタ 必要ない場合はNULLを指定する
 +
 + Returns:
 +  status_vector[1]の値を返します
 +  成功した場合は0となります
 +  失敗した場合は非0となります
 +  また、エラーが発生した場合はstatus_vector[0]を1にセットし、status_vector[1]にエラーコードをセットします
 +/
ISC_STATUS isc_dsql_prepare(ISC_STATUS* status_vector, isc_tr_handle* trans_handle, isc_stmt_handle* stmt_handle, ushort length, const(char)* statement, ushort dialect, XSQLDA* xsqlda);

ISC_STATUS isc_dsql_set_cursor_name(ISC_STATUS* a, isc_stmt_handle* b, const(char)* c, ushort d);

ISC_STATUS isc_dsql_sql_info(ISC_STATUS* a, isc_stmt_handle* b, short c, const(char)* d, short e, char* f);

void isc_encode_date(const(void)* a, ISC_QUAD* b);

void isc_encode_sql_date(const(void)* a, ISC_DATE* b);

void isc_encode_sql_time(const(void)* a, ISC_TIME* b);

void isc_encode_timestamp(const(void)* a, ISC_TIMESTAMP* b);

int isc_event_block(ubyte** a, ubyte** b, ushort c, ...);

void isc_event_counts(uint* a, short b, ubyte* c, const(ubyte)* d);

/+
 + 非推奨のため、isc_modify_dpbを使用すること
 +/
deprecated void isc_expand_dpb(char** a, short* b, ...);

/++
 + DPBを設定する
 +
 + DPB(DatabaseParameterBuffer)とは、データベース接続時の設定情報のバッファ
 + この情報をtype指定して一つずつ設定を行う
 +
 + Params:
 +  dpb = DPBのポインタ
 +  dpb_size = dpbの長さを格納しているデータのポインタ ここに長さが書き込まれる
 +  type = 変更するデータの種類を指定するisc_user_nameとかisc_dpb_passwordとか
 +  string = typeで指定した種類に対するデータを指定する
 +  string_length = stringの長さを指定する
 +
 + Returns:
 +  成功した場合は0、失敗した場合は非0を返す
 +/
int isc_modify_dpb(ubyte** dpb, short* dpb_size, ushort type, const(char)* string, short string_length);

int isc_free(char* a);

ISC_STATUS isc_get_slice(ISC_STATUS* a, isc_db_handle* b, isc_tr_handle* c, ISC_QUAD* d, short e, const(char)* f, short g, const(int)* h, int i, void* j, int* k);

/* CVC: This non-const signature is needed for compatibility, see gds.cpp. */
deprecated int isc_interprete(char* a, ISC_STATUS** b);

/++
 + STATUS_VECTORを元にメッセージを取得
 +
 + isc_interpreteの代わりにこちらを使用してください
 +
 + Params:
 +  buffer = メッセージを書き込むバッファ
 +  buffer_length = bufferで指定したバッファのサイズ
 +  status_vector = 元データが入ったSTATUS_VECTORのポインタのポインタ
 +
 + Returns:
 +  実際にbufferに書き込んだサイズ
 +/
int fb_interpret(char* buffer, uint buffer_length, const(ISC_STATUS*)* status_vector);

ISC_STATUS isc_prepare_transaction2(ISC_STATUS* a, isc_tr_handle* b, ushort c, const(ubyte)* d);

void isc_print_sqlerror(short a, const(ISC_STATUS)* b);

ISC_STATUS isc_print_status(const(ISC_STATUS)* a);

ISC_STATUS isc_put_slice(ISC_STATUS* a, isc_db_handle* b, isc_tr_handle* c, ISC_QUAD* d, short e, const(char)* f, short g, const(int)* h, int i, void* j);

ISC_STATUS isc_que_events(ISC_STATUS* a, isc_db_handle* b, int* c, short d, const(ubyte)* e, ISC_EVENT_CALLBACK f, void* g);

ISC_STATUS isc_rollback_retaining(ISC_STATUS* a, isc_tr_handle* b);

/++
 + トランザクションを開始する
 +
 + 第4以降の引数は、データベースハンドル、TPBの長さ、TPBのポインタを1セットとし、
 + db_handle_countで指定した数だけ指定する
 + TPBとはTransactionParameterBufferの略で、トランザクションの制御を指定するパラメータ
 +
 + Params:
 +  status_vector = エラーステータスを受け取るISC_STATUS配列へのポインタ
 +  trans_handle = トランザクションハンドルを受け取るポインタ
 +  db_handle_count = トランザクションで使用するデータベースの数
 +
 + Returns:
 +  成功した場合は0、失敗した場合は非0を返す
 +/
ISC_STATUS isc_start_transaction(ISC_STATUS* status, isc_tr_handle* trans_handle, short db_handle_count, ...);

/++
 + トランザクションを開始する
 +
 + この関数が用意されているのは下記の理由
 + 1. 可変長配列が使えない言語から使用するときのため
 + 2. プログラム作成時に接続先データベースが確定していない時のため
 +
 + Params:
 +  status_vector = エラーステータスを受け取るISC_STATUS配列へのポインタ
 +  trans_handle = トランザクションハンドルを受け取るポインタ
 +  db_handle_count = トランザクションで使用するデータベースの数
 +  teb_vector_address = データベース情報を書き込んだ構造体の配列へのポインタ
 +
 + Returns:
 +  成功した場合は0、失敗した場合は非0を返す
 +/
ISC_STATUS isc_start_multiple(ISC_STATUS* status_vector, isc_tr_handle* trans_handle, short db_handle_count, ISC_TEB* teb_vector_address);

/++
 + トランザクションをロールバックする
 +
 + Params:
 +  status_vector = エラーステータスを受け取るISC_STATUS配列へのポインタ
 +  trans_handle = トランザクションハンドルのポインタ
 +
 + Returns:
 +  成功した場合は0、失敗した場合は非0を返す
 +/
ISC_STATUS isc_rollback_transaction(ISC_STATUS* status_vector, isc_tr_handle* trans_handle);

/++
 + トランザクションをコミットする
 +
 + Params:
 +  status = エラーステータスを受け取るISC_STATUS配列へのポインタ
 +  trans_handle = トランザクションハンドルのポインタ
 +
 + Returns:
 +  成功した場合は0、失敗した場合は非0を返す
 +/
ISC_STATUS isc_commit_transaction(ISC_STATUS* status, isc_tr_handle* trans_handle);

int isc_sqlcode(const(ISC_STATUS)* a);

void isc_sql_interprete(short a, char* b, short c);

ISC_STATUS isc_transaction_info(ISC_STATUS* a, isc_tr_handle* b, short c, const(char)* d, short e, char* f);

ISC_STATUS isc_transact_request(ISC_STATUS* a, isc_db_handle* b, isc_tr_handle* c, ushort d, const(char)* e, ushort f, char* g, ushort h, char* i);

int isc_vax_integer(const(char)* a, short b);

long isc_portable_integer(const(ubyte)* a, short b);

} // extern (C)

/*************************************/
/* Security Functions and structures */
/*************************************/

immutable int sec_uid_spec = 0x01;
immutable int sec_gid_spec = 0x02;
immutable int sec_server_spec = 0x04;
immutable int sec_password_spec = 0x08;
immutable int sec_group_name_spec = 0x10;
immutable int sec_first_name_spec = 0x20;
immutable int sec_middle_name_spec = 0x40;
immutable int sec_last_name_spec = 0x80;
immutable int sec_dba_user_name_spec = 0x100;
immutable int sec_dba_password_spec = 0x200;

immutable int sec_protocol_tcpip = 1;
immutable int sec_protocol_netbeui = 2;
immutable int sec_protocol_spx = 3; /* -- Deprecated Protocol. Declaration retained for compatibility   */
immutable int sec_protocol_local = 4;

struct USER_SEC_DATA
{
	short sec_flags;    /* which fields are specified */
	int uid;      /* the user's id */
	int gid;      /* the user's group id */
	int protocol;     /* protocol to use for connection */
	char* server;    /* server to administer */
	char* user_name;   /* the user's name */
	char* password;   /* the user's password */
	char* group_name;   /* the group name */
	char* first_name;   /* the user's first name */
	char* middle_name;  /* the user's middle name */
	char* last_name;   /* the user's last name */
	char* dba_user_name;  /* the dba user name */
	char* dba_password;  /* the dba password */
}

extern (C)
{
ISC_STATUS isc_add_user(ISC_STATUS* a, const(USER_SEC_DATA)* b);

ISC_STATUS isc_delete_user(ISC_STATUS* a, const(USER_SEC_DATA)* b);

ISC_STATUS isc_modify_user(ISC_STATUS* a, const(USER_SEC_DATA)* b);
}
/**********************************/
/*  Other OSRI functions          */
/**********************************/
extern (C)
{
ISC_STATUS isc_compile_request(ISC_STATUS* a, isc_db_handle* b, isc_req_handle* c, short d, const(char)* e);

ISC_STATUS isc_compile_request2(ISC_STATUS* a, isc_db_handle* b, isc_req_handle* c, short d, const(char)* e);

ISC_STATUS isc_ddl(ISC_STATUS* a, isc_db_handle* b, isc_tr_handle* c, short d, const(char)* e);

ISC_STATUS isc_prepare_transaction(ISC_STATUS* a, isc_tr_handle* b);


ISC_STATUS isc_receive(ISC_STATUS* a, isc_req_handle* b, short c, short d, void* e, short f);

ISC_STATUS isc_reconnect_transaction(ISC_STATUS* a, isc_db_handle* b, isc_tr_handle* c, short d, const(char)* e);

ISC_STATUS isc_release_request(ISC_STATUS* a, isc_req_handle* b);

ISC_STATUS isc_request_info(ISC_STATUS* a, isc_req_handle* b, short c, short d, const(char)* e, short f, char* g);

ISC_STATUS isc_seek_blob(ISC_STATUS* a, isc_blob_handle* b, short c, int d, int* e);

ISC_STATUS isc_send(ISC_STATUS* a, isc_req_handle* b, short c, short d, const(void)* e, short f);

ISC_STATUS isc_start_and_send(ISC_STATUS* a, isc_req_handle* b, isc_tr_handle* c, short d, short e, const(void)* f, short g);

ISC_STATUS isc_start_request(ISC_STATUS* a, isc_req_handle* b, isc_tr_handle* c, short d);

ISC_STATUS isc_unwind_request(ISC_STATUS* a, isc_tr_handle* b, short c);

ISC_STATUS isc_wait_for_event(ISC_STATUS* a, isc_db_handle* b, short c, const(ubyte)* d, ubyte* e);
} // extern (C)

/*****************************/
/* Other Sql functions       */
/*****************************/

extern (C)
{
ISC_STATUS isc_close(ISC_STATUS* a, const(char)* b);

ISC_STATUS isc_declare(ISC_STATUS* a, const(char)* b, const(char)* c);

ISC_STATUS isc_describe(ISC_STATUS* a, const(char)* b, XSQLDA* c);

ISC_STATUS isc_describe_bind(ISC_STATUS* a, const(char)* b, XSQLDA* c);

ISC_STATUS isc_execute(ISC_STATUS* a, isc_tr_handle* b, const(char)* c, XSQLDA* d);

ISC_STATUS isc_execute_immediate(ISC_STATUS* status, isc_db_handle* databaseHandle, isc_tr_handle* transactionHandle, short* num, const(char)* e);

ISC_STATUS isc_fetch(ISC_STATUS* a, const(char)* b, XSQLDA* c);

ISC_STATUS isc_open(ISC_STATUS* a, isc_tr_handle* b, const(char)* c, XSQLDA* d);

ISC_STATUS isc_prepare(ISC_STATUS* a, isc_db_handle* b, isc_tr_handle* c, const(char)* d, short* e, const(char)* f, XSQLDA* g);
} // extern (C)

/*************************************/
/* Other Dynamic sql functions       */
/*************************************/

extern (C)
{
ISC_STATUS isc_dsql_execute_m(ISC_STATUS* a, isc_tr_handle* b, isc_stmt_handle* c, ushort d, const(char)* e, ushort f, ushort g, char* h);

ISC_STATUS isc_dsql_execute2_m(ISC_STATUS* a, isc_tr_handle* b, isc_stmt_handle* c, ushort d, const(char)* e, ushort f, ushort g, const(char)* h, ushort i, char* j, ushort k, ushort l, char* m);

ISC_STATUS isc_dsql_execute_immediate_m(ISC_STATUS* a, isc_db_handle* b, isc_tr_handle* c, ushort d, const(char)* e, ushort f, ushort g, const(char)* h, ushort i, ushort j, char* k);

ISC_STATUS isc_dsql_exec_immed3_m(ISC_STATUS* a, isc_db_handle* b, isc_tr_handle* c, ushort d, const(char)* e, ushort f, ushort g, const(char)* h, ushort i, ushort j, char* k, ushort l, char* m, ushort n, ushort o, char* p);

ISC_STATUS isc_dsql_fetch_m(ISC_STATUS* a, isc_stmt_handle* b, ushort c, const(char)* d, ushort e, ushort f, char* g);

ISC_STATUS isc_dsql_insert_m(ISC_STATUS* a, isc_stmt_handle* b, ushort c, const(char)* d, ushort e, ushort f, char* g);

ISC_STATUS isc_dsql_prepare_m(ISC_STATUS* a, isc_tr_handle* b, isc_stmt_handle* c, ushort d, const(char)* e, ushort f, ushort g, const(char)* h, ushort i, char* j);

ISC_STATUS isc_dsql_release(ISC_STATUS* a, const(char)* b);

ISC_STATUS isc_embed_dsql_close(ISC_STATUS* a, const(char)* b);

ISC_STATUS isc_embed_dsql_declare(ISC_STATUS* a, const(char)* b, const(char)* c);

ISC_STATUS isc_embed_dsql_describe(ISC_STATUS* a, const(char)* b, ushort c, XSQLDA* d);

ISC_STATUS isc_embed_dsql_describe_bind(ISC_STATUS* a, const(char)* b, ushort c, XSQLDA* d);

ISC_STATUS isc_embed_dsql_execute(ISC_STATUS* a, isc_tr_handle* b, const(char)* c, ushort d, XSQLDA* e);

ISC_STATUS isc_embed_dsql_execute2(ISC_STATUS* a, isc_tr_handle* b, const(char)* c, ushort d, XSQLDA* e, XSQLDA* f);

ISC_STATUS isc_embed_dsql_execute_immed(ISC_STATUS* a, isc_db_handle* b, isc_tr_handle* c, ushort d, const(char)* e, ushort f, XSQLDA* g);

ISC_STATUS isc_embed_dsql_fetch(ISC_STATUS* a, const(char)* b, ushort c, XSQLDA* d);

ISC_STATUS isc_embed_dsql_open(ISC_STATUS* a, isc_tr_handle* b, const(char)* c, ushort d, XSQLDA* e);

ISC_STATUS isc_embed_dsql_open2(ISC_STATUS* a, isc_tr_handle* b, const(char)* c, ushort d, XSQLDA* e, XSQLDA* f);

ISC_STATUS isc_embed_dsql_insert(ISC_STATUS* a, const(char)* b, ushort c, XSQLDA* d);

ISC_STATUS isc_embed_dsql_prepare(ISC_STATUS* a, isc_db_handle* b, isc_tr_handle* c, const(char)* d, ushort e, const(char)* f, ushort g, XSQLDA* h);

ISC_STATUS isc_embed_dsql_release(ISC_STATUS* a, const(char)* b);
} // extern (C)

/******************************/
/* Other Blob functions       */
/******************************/
extern (C)
{
BSTREAM* BLOB_open(isc_blob_handle a, char* b, int c);

int BLOB_put(char a, BSTREAM* b);

int BLOB_close(BSTREAM* a);

int BLOB_get(BSTREAM* a);

int BLOB_display(ISC_QUAD* a, isc_db_handle b, isc_tr_handle c, const(char)* d);

int BLOB_dump(ISC_QUAD* a, isc_db_handle b, isc_tr_handle c, const(char)* d);

int BLOB_edit(ISC_QUAD* a, isc_db_handle b, isc_tr_handle c, const(char)* d);

int BLOB_load(ISC_QUAD* a, isc_db_handle b, isc_tr_handle c, const(char)* d);

int BLOB_text_dump(ISC_QUAD* a, isc_db_handle b, isc_tr_handle c, const(char)* d);

int BLOB_text_load(ISC_QUAD* a, isc_db_handle b, isc_tr_handle c, const(char)* d);

BSTREAM* Bopen(ISC_QUAD* a, isc_db_handle b, isc_tr_handle c, const(char)* d);
} // extern (C)

/******************************/
/* Other Misc functions       */
/******************************/
extern (C)
{
int isc_ftof(const(char)* a, ushort b, char* c, ushort d);

ISC_STATUS isc_print_blr(const(char)* a, ISC_PRINT_CALLBACK b, void* c, short d);

void isc_set_debug(int a);

void isc_qtoq(const(ISC_QUAD)* a, ISC_QUAD* b);

void isc_vtof(const(char)* a, char* b, ushort c);

void isc_vtov(const(char)* a, char* b, short c);

int isc_version(isc_db_handle* a, ISC_VERSION_CALLBACK b, void* c);

int isc_reset_fpe(ushort a);
} // extern (C)

/*****************************************/
/* Service manager functions             */
/*****************************************/
void ADD_SPB_LENGTH(char* p, ushort length)
{
	*p++ = cast(char)length;
	*p++ = cast(char)(length >> 8);
}

void ADD_SPB_NUMERIC(char* p, uint data)
{
	*p++ = cast(char)data;
	*p++ = cast(char)(data >> 8);
	*p++ = cast(char)(data >> 16);
	*p++ = cast(char)(data >> 24);
}

extern (C)
{
ISC_STATUS isc_service_attach(ISC_STATUS* a, ushort b, const(char)* c, isc_svc_handle* d, ushort e, const(char)* f);

ISC_STATUS isc_service_detach(ISC_STATUS* a, isc_svc_handle* b);

ISC_STATUS isc_service_query(ISC_STATUS* a, isc_svc_handle* b, isc_resv_handle* c, ushort d, const(char)* e, ushort f, const(char)* g, ushort h, char* i);

ISC_STATUS isc_service_start(ISC_STATUS* a, isc_svc_handle* b, isc_resv_handle* c, ushort d, const(char)* e);
} // extern (C)

/********************************/
/* Client information functions */
/********************************/
extern (C)
{
void isc_get_client_version (char* a);
int  isc_get_client_major_version ();
int  isc_get_client_minor_version ();
} // extern (C)


/***************************************************/
/* Actions to pass to the blob filter (ctl_source) */
/***************************************************/

immutable int isc_blob_filter_open = 0;
immutable int isc_blob_filter_get_segment = 1;
immutable int isc_blob_filter_close = 2;
immutable int isc_blob_filter_create = 3;
immutable int isc_blob_filter_put_segment = 4;
immutable int isc_blob_filter_alloc = 5;
immutable int isc_blob_filter_free = 6;
immutable int isc_blob_filter_seek = 7;

/*******************/
/* Blr definitions */
/*******************/



/*  WARNING: if you add a new BLR representing a data type, and the value
 *           is greater than the numerically greatest value which now
 *           represents a data type, you must change the define for
 *           DTYPE_BLR_MAX in jrd/align.h, and add the necessary entries
 *           to all the arrays in that file.
 */

immutable int blr_text = 14;
immutable int blr_text2 = 15; /* added in 3.2 JPN */
immutable int blr_short = 7;
immutable int blr_long = 8;
immutable int blr_quad = 9;
immutable int blr_float = 10;
immutable int blr_double = 27;
immutable int blr_d_float = 11;
immutable int blr_timestamp = 35;
immutable int blr_varying = 37;
immutable int blr_varying2 = 38; /* added in 3.2 JPN */
immutable int blr_blob = 261;
immutable int blr_cstring = 40;      
immutable int blr_cstring2 = 41; /* added in 3.2 JPN */
immutable int blr_blob_id = 45; /* added from gds.h */
immutable int blr_sql_date = 12;
immutable int blr_sql_time = 13;
immutable int blr_int64 = 16;
immutable int blr_blob2 = 17;

/* Historical alias for pre V6 applications */
immutable int blr_date = blr_timestamp;

immutable int blr_inner = 0;
immutable int blr_left = 1;
immutable int blr_right = 2;
immutable int blr_full = 3;

immutable int blr_gds_code = 0;
immutable int blr_sql_code = 1;
immutable int blr_exception = 2;
immutable int blr_trigger_code = 3;
immutable int blr_default_code = 4;
immutable int blr_raise = 5;
immutable int blr_exception_msg = 6;

immutable int blr_version4 = 4;
immutable int blr_version5 = 5;
immutable int blr_eoc = 76;
immutable int blr_end = 255; /* note: defined as -1 in gds.h */

immutable int blr_assignment = 1;
immutable int blr_begin = 2;
immutable int blr_dcl_variable = 3; /* added from gds.h */
immutable int blr_message = 4;
immutable int blr_erase = 5;
immutable int blr_fetch = 6;
immutable int blr_for = 7;
immutable int blr_if = 8;
immutable int blr_loop = 9;
immutable int blr_modify = 10;
immutable int blr_handler = 11;
immutable int blr_receive = 12;
immutable int blr_select = 13;
immutable int blr_send = 14;
immutable int blr_store = 15;
immutable int blr_label = 17;
immutable int blr_leave = 18;
immutable int blr_store2 = 19;
immutable int blr_post = 20;
immutable int blr_literal = 21;
immutable int blr_dbkey = 22;
immutable int blr_field = 23;
immutable int blr_fid = 24;
immutable int blr_parameter = 25;
immutable int blr_variable = 26;
immutable int blr_average = 27;
immutable int blr_count = 28;
immutable int blr_maximum = 29;
immutable int blr_minimum = 30;
immutable int blr_total = 31;
immutable int blr_add = 34;
immutable int blr_subtract = 35;
immutable int blr_multiply = 36;
immutable int blr_divide = 37;
immutable int blr_negate = 38;
immutable int blr_concatenate = 39;
immutable int blr_substring = 40;
immutable int blr_parameter2 = 41;
immutable int blr_from = 42;
immutable int blr_via = 43;
immutable int blr_parameter2_old = 44; /* Confusion */
immutable int blr_user_name = 44; /* added from gds.h */
immutable int blr_null = 45;
immutable int blr_equiv = 46;
immutable int blr_eql = 47;
immutable int blr_neq = 48;
immutable int blr_gtr = 49;
immutable int blr_geq = 50;
immutable int blr_lss = 51;
immutable int blr_leq = 52;
immutable int blr_containing = 53;
immutable int blr_matching = 54;
immutable int blr_starting = 55;
immutable int blr_between = 56;
immutable int blr_or = 57;
immutable int blr_and = 58;
immutable int blr_not = 59;
immutable int blr_any = 60;
immutable int blr_missing = 61;
immutable int blr_unique = 62;
immutable int blr_like = 63;
immutable int blr_rse = 67;
immutable int blr_first = 68;
immutable int blr_project = 69;
immutable int blr_sort = 70;
immutable int blr_boolean = 71;
immutable int blr_ascending = 72;
immutable int blr_descending = 73;
immutable int blr_relation = 74;
immutable int blr_rid = 75;
immutable int blr_union = 76;
immutable int blr_map = 77;
immutable int blr_group_by = 78;
immutable int blr_aggregate = 79;
immutable int blr_join_type = 80;
immutable int blr_agg_count = 83;
immutable int blr_agg_max = 84;
immutable int blr_agg_min = 85;
immutable int blr_agg_total = 86;
immutable int blr_agg_average = 87;
immutable int blr_parameter3 = 88; /* same as Rdb definition */
immutable int blr_run_max = 89;
immutable int blr_run_min = 90;
immutable int blr_run_total = 91;
immutable int blr_run_average = 92;
immutable int blr_agg_count2 = 93;
immutable int blr_agg_count_distinct = 94;
immutable int blr_agg_total_distinct = 95;
immutable int blr_agg_average_distinct = 96;
immutable int blr_function = 100;
immutable int blr_gen_id = 101;
immutable int blr_prot_mask = 102;
immutable int blr_upcase = 103;
immutable int blr_lock_state = 104;
immutable int blr_value_if = 105;
immutable int blr_matching2 = 106;
immutable int blr_index = 107;
immutable int blr_ansi_like = 108;
immutable int blr_seek = 112;
immutable int blr_run_count = 118; /* changed from 88 to avoid conflict with blr_parameter3 */
immutable int blr_rs_stream = 119;
immutable int blr_exec_proc = 120;
immutable int blr_procedure = 124;
immutable int blr_pid = 125;
immutable int blr_exec_pid = 126;
immutable int blr_singular = 127;
immutable int blr_abort = 128;
immutable int blr_block = 129;
immutable int blr_error_handler = 130;
immutable int blr_cast = 131;
immutable int blr_start_savepoint = 134;
immutable int blr_end_savepoint = 135;
immutable int blr_plan = 139; /* access plan items */
immutable int blr_merge = 140;
immutable int blr_join = 141;
immutable int blr_sequential = 142;
immutable int blr_navigational = 143;
immutable int blr_indices = 144;
immutable int blr_retrieve = 145;
immutable int blr_relation2 = 146;
immutable int blr_rid2 = 147;
immutable int blr_set_generator = 150;
immutable int blr_ansi_any = 151;   /* required for NULL handling */
immutable int blr_exists = 152;   /* required for NULL handling */
immutable int blr_record_version = 154; /* get tid of record */
immutable int blr_stall = 155; /* fake server stall */
immutable int blr_ansi_all = 158;   /* required for NULL handling */
immutable int blr_extract = 159;
immutable int blr_current_date = 160;
immutable int blr_current_timestamp = 161;
immutable int blr_current_time = 162;
/* These codes reuse BLR code space */

immutable int blr_post_arg = 163;
immutable int blr_exec_into = 164;
immutable int blr_user_savepoint = 165;
immutable int blr_dcl_cursor = 166;
immutable int blr_cursor_stmt = 167;
immutable int blr_current_timestamp2 = 168;
immutable int blr_current_time2 = 169;
/* FB 1.0 specific BLR */

immutable int blr_current_role = 174;
immutable int blr_skip = 175;

/* FB 1.5 specific BLR */

immutable int blr_exec_sql = 176;
immutable int blr_internal_info = 177;
immutable int blr_nullsfirst = 178;
immutable int blr_writelock = 179;
immutable int blr_nullslast = 180;

/* FB 2.0 specific BLR */

immutable int blr_lowcase = 181;
immutable int blr_strlen = 182;

immutable int blr_trim = 183;


/* these indicate directions for blr_seek and blr_find */
immutable int blr_continue = 0;
immutable int blr_forward = 1;
immutable int blr_backward = 2;
immutable int blr_bof_forward = 3;
immutable int blr_eof_backward = 4;

/* sub parameters for blr_extract */

immutable int blr_extract_year = 0;
immutable int blr_extract_month = 1;
immutable int blr_extract_day = 2;
immutable int blr_extract_hour = 3;
immutable int blr_extract_minute = 4;
immutable int blr_extract_second = 5;
immutable int blr_extract_weekday = 6;
immutable int blr_extract_yearday = 7;

/* sub parameters for blr_length */
immutable int blr_strlen_bit = 0;
immutable int blr_strlen_char = 1;
immutable int blr_strlen_octet = 2;

/* sub parameters for blr_trim */
immutable int blr_trim_both = 0;
immutable int blr_trim_leading = 1;
immutable int blr_trim_trailing = 2;
immutable int blr_trim_spaces = 0;
immutable int blr_trim_characters = 1;

/* These codes are actions for user-defined savepoints */

immutable int blr_savepoint_set = 0;
immutable int blr_savepoint_release = 1;
immutable int blr_savepoint_undo = 2;
immutable int blr_savepoint_release_single = 3;

/* These codes are actions for cursors */

immutable int blr_cursor_open = 0;
immutable int blr_cursor_close = 1;
immutable int blr_cursor_fetch = 2;


/**********************************/
/* Database parameter block stuff */
/**********************************/

immutable int isc_dpb_version1 = 1;
immutable int isc_dpb_cdd_pathname = 1;
immutable int isc_dpb_allocation = 2;
immutable int isc_dpb_journal = 3;
immutable int isc_dpb_page_size = 4;
immutable int isc_dpb_num_buffers = 5;
immutable int isc_dpb_buffer_length = 6;
immutable int isc_dpb_debug = 7;
immutable int isc_dpb_garbage_collect = 8;
immutable int isc_dpb_verify = 9;
immutable int isc_dpb_sweep = 10;
immutable int isc_dpb_enable_journal = 11;
immutable int isc_dpb_disable_journal = 12;
immutable int isc_dpb_dbkey_scope = 13;
immutable int isc_dpb_number_of_users = 14;
immutable int isc_dpb_trace = 15;
immutable int isc_dpb_no_garbage_collect = 16;
immutable int isc_dpb_damaged = 17;
immutable int isc_dpb_license = 18;
immutable int isc_dpb_sys_user_name = 19;
immutable int isc_dpb_encrypt_key = 20;
immutable int isc_dpb_activate_shadow = 21;
immutable int isc_dpb_sweep_interval = 22;
immutable int isc_dpb_delete_shadow = 23;
immutable int isc_dpb_force_write = 24;
immutable int isc_dpb_begin_log = 25;
immutable int isc_dpb_quit_log = 26;
immutable int isc_dpb_no_reserve = 27;
immutable int isc_dpb_user_name = 28;
immutable int isc_dpb_password = 29;
immutable int isc_dpb_password_enc = 30;
immutable int isc_dpb_sys_user_name_enc = 31;
immutable int isc_dpb_interp = 32;
immutable int isc_dpb_online_dump = 33;
immutable int isc_dpb_old_file_size = 34;
immutable int isc_dpb_old_num_files = 35;
immutable int isc_dpb_old_file = 36;
immutable int isc_dpb_old_start_page = 37;
immutable int isc_dpb_old_start_seqno = 38;
immutable int isc_dpb_old_start_file = 39;
immutable int isc_dpb_drop_walfile = 40;
immutable int isc_dpb_old_dump_id = 41;
immutable int isc_dpb_wal_backup_dir = 42;
immutable int isc_dpb_wal_chkptlen = 43;
immutable int isc_dpb_wal_numbufs = 44;
immutable int isc_dpb_wal_bufsize = 45;
immutable int isc_dpb_wal_grp_cmt_wait = 46;
immutable int isc_dpb_lc_messages = 47;
immutable int isc_dpb_lc_ctype = 48;
immutable int isc_dpb_cache_manager = 49;
immutable int isc_dpb_shutdown = 50;
immutable int isc_dpb_online = 51;
immutable int isc_dpb_shutdown_delay = 52;
immutable int isc_dpb_reserved = 53;
immutable int isc_dpb_overwrite = 54;
immutable int isc_dpb_sec_attach = 55;
immutable int isc_dpb_disable_wal = 56;
immutable int isc_dpb_connect_timeout = 57;
immutable int isc_dpb_dummy_packet_interval = 58;
immutable int isc_dpb_gbak_attach = 59;
immutable int isc_dpb_sql_role_name = 60;
immutable int isc_dpb_set_page_buffers = 61;
immutable int isc_dpb_working_directory = 62;
immutable int isc_dpb_sql_dialect = 63;
immutable int isc_dpb_set_db_readonly = 64;
immutable int isc_dpb_set_db_sql_dialect = 65;
immutable int isc_dpb_gfix_attach = 66;
immutable int isc_dpb_gstat_attach = 67;
immutable int isc_dpb_set_db_charset = 68;
immutable int isc_dpb_gsec_attach = 69;
immutable int isc_dpb_address_path = 70;

/**************************************************/
/* clumplet tags used inside isc_dpb_address_path */
/*       and isc_spb_address_path */
/**************************************************/

/* Format of this clumplet is the following:

 <address-path-clumplet> ::=
	isc_dpb_address_path <byte-clumplet-length> <address-stack>

 <address-stack> ::=
	<address-descriptor> |
	<address-stack> <address-descriptor>

 <address-descriptor> ::=
	isc_dpb_address <byte-clumplet-length> <address-elements>

 <address-elements> ::=
	<address-element> |
	<address-elements> <address-element>

 <address-element> ::=
	isc_dpb_addr_protocol <byte-clumplet-length> <protocol-string> |
	isc_dpb_addr_endpoint <byte-clumplet-length> <remote-endpoint-string>

 <protocol-string> ::=
	"TCPv4" |
	"TCPv6" |
	"XNET" |
	"WNET" |
	....

 <remote-endpoint-string> ::=
	<IPv4-address> | // such as "172.20.1.1"
	<IPv6-address> | // such as "2001:0:13FF:09FF::1"
	<xnet-process-id> | // such as "17864"
	...
*/

immutable int isc_dpb_address = 1;

immutable int isc_dpb_addr_protocol = 1;
immutable int isc_dpb_addr_endpoint = 2;

/*********************************/
/* isc_dpb_verify specific flags */
/*********************************/

immutable int isc_dpb_pages = 1;
immutable int isc_dpb_records = 2;
immutable int isc_dpb_indices = 4;
immutable int isc_dpb_transactions = 8;
immutable int isc_dpb_no_update = 16;
immutable int isc_dpb_repair = 32;
immutable int isc_dpb_ignore = 64;

/***********************************/
/* isc_dpb_shutdown specific flags */
/***********************************/

immutable int isc_dpb_shut_cache = 0x1;
immutable int isc_dpb_shut_attachment = 0x2;
immutable int isc_dpb_shut_transaction = 0x4;
immutable int isc_dpb_shut_force = 0x8;
immutable int isc_dpb_shut_mode_mask = 0x70;

immutable int isc_dpb_shut_default = 0x0;
immutable int isc_dpb_shut_normal = 0x10;
immutable int isc_dpb_shut_multi = 0x20;
immutable int isc_dpb_shut_single = 0x30;
immutable int isc_dpb_shut_full = 0x40;

/**************************************/
/* Bit assignments in RDB$SYSTEM_FLAG */
/**************************************/

immutable int RDB_system = 1;
immutable int RDB_id_assigned = 2;
/* 2 is for QLI. See jrd/constants.h for more Firebird-specific values. */


/*************************************/
/* Transaction parameter block stuff */
/*************************************/

immutable byte isc_tpb_version1 = 1;
immutable byte isc_tpb_version3 = 3;
immutable byte isc_tpb_consistency = 1;
immutable byte isc_tpb_concurrency = 2;
immutable byte isc_tpb_shared = 3;
immutable byte isc_tpb_protected = 4;
immutable byte isc_tpb_exclusive = 5;
immutable byte isc_tpb_wait = 6;
immutable byte isc_tpb_nowait = 7;
immutable byte isc_tpb_read = 8;
immutable byte isc_tpb_write = 9;
immutable byte isc_tpb_lock_read = 10;
immutable byte isc_tpb_lock_write = 11;
immutable byte isc_tpb_verb_time = 12;
immutable byte isc_tpb_commit_time = 13;
immutable byte isc_tpb_ignore_limbo = 14;
immutable byte isc_tpb_read_committed = 15;
immutable byte isc_tpb_autocommit = 16;
immutable byte isc_tpb_rec_version = 17;
immutable byte isc_tpb_no_rec_version = 18;
immutable byte isc_tpb_restart_requests = 19;
immutable byte isc_tpb_no_auto_undo = 20;
immutable byte isc_tpb_lock_timeout = 21;


/************************/
/* Blob Parameter Block */
/************************/

immutable int isc_bpb_version1 = 1;
immutable int isc_bpb_source_type = 1;
immutable int isc_bpb_target_type = 2;
immutable int isc_bpb_type = 3;
immutable int isc_bpb_source_interp = 4;
immutable int isc_bpb_target_interp = 5;
immutable int isc_bpb_filter_parameter = 6;

immutable int isc_bpb_type_segmented = 0;
immutable int isc_bpb_type_stream = 1;


/*********************************/
/* Service parameter block stuff */
/*********************************/

immutable int isc_spb_version1 = 1;
immutable int isc_spb_current_version = 2;
immutable int isc_spb_version = isc_spb_current_version;
immutable int isc_spb_user_name = isc_dpb_user_name;
immutable int isc_spb_sys_user_name = isc_dpb_sys_user_name;
immutable int isc_spb_sys_user_name_enc = isc_dpb_sys_user_name_enc;
immutable int isc_spb_password = isc_dpb_password;
immutable int isc_spb_password_enc = isc_dpb_password_enc;
immutable int isc_spb_command_line = 105;
immutable int isc_spb_dbname = 106;
immutable int isc_spb_verbose = 107;
immutable int isc_spb_options = 108;
immutable int isc_spb_address_path = 109;


immutable int isc_spb_connect_timeout = isc_dpb_connect_timeout;
immutable int isc_spb_dummy_packet_interval = isc_dpb_dummy_packet_interval;
immutable int isc_spb_sql_role_name = isc_dpb_sql_role_name;


/*********************************/
/* Information call declarations */
/*********************************/

/* Common, structural codes */
/****************************/

immutable int isc_info_end = 1;
immutable int isc_info_truncated = 2;
immutable int isc_info_error = 3;
immutable int isc_info_data_not_ready = 4;
immutable int isc_info_flag_end = 127;

/******************************/
/* Database information items */
/******************************/

enum db_info_types
{
	isc_info_db_id   = 4,
	isc_info_reads   = 5,
	isc_info_writes      = 6,
	isc_info_fetches  = 7,
	isc_info_marks   = 8,

	isc_info_implementation = 11,
	isc_info_isc_version  = 12,
	isc_info_base_level  = 13,
	isc_info_page_size  = 14,
	isc_info_num_buffers = 15,
	isc_info_limbo   = 16,
	isc_info_current_memory = 17,
	isc_info_max_memory  = 18,
	isc_info_window_turns = 19,
	isc_info_license  = 20,

	isc_info_allocation  = 21,
	isc_info_attachment_id  = 22,
	isc_info_read_seq_count = 23,
	isc_info_read_idx_count = 24,
	isc_info_insert_count  = 25,
	isc_info_update_count  = 26,
	isc_info_delete_count  = 27,
	isc_info_backout_count   = 28,
	isc_info_purge_count  = 29,
	isc_info_expunge_count  = 30,

	isc_info_sweep_interval = 31,
	isc_info_ods_version  = 32,
	isc_info_ods_minor_version = 33,
	isc_info_no_reserve  = 34,
	/* Begin deprecated WAL and JOURNAL items. */
	isc_info_logfile  = 35,
	isc_info_cur_logfile_name = 36,
	isc_info_cur_log_part_offset = 37,
	isc_info_num_wal_buffers = 38,
	isc_info_wal_buffer_size = 39,
	isc_info_wal_ckpt_length = 40,

	isc_info_wal_cur_ckpt_interval = 41,
	isc_info_wal_prv_ckpt_fname = 42,
	isc_info_wal_prv_ckpt_poffset = 43,
	isc_info_wal_recv_ckpt_fname = 44,
	isc_info_wal_recv_ckpt_poffset = 45,
	isc_info_wal_grpc_wait_usecs = 47,
	isc_info_wal_num_io  = 48,
	isc_info_wal_avg_io_size = 49,
	isc_info_wal_num_commits = 50,
	isc_info_wal_avg_grpc_size = 51,
	/* End deprecated WAL and JOURNAL items. */

	isc_info_forced_writes  = 52,
	isc_info_user_names = 53,
	isc_info_page_errors = 54,
	isc_info_record_errors = 55,
	isc_info_bpage_errors = 56,
	isc_info_dpage_errors = 57,
	isc_info_ipage_errors = 58,
	isc_info_ppage_errors = 59,
	isc_info_tpage_errors = 60,

	isc_info_set_page_buffers = 61,
	isc_info_db_sql_dialect = 62,   
	isc_info_db_read_only = 63,
	isc_info_db_size_in_pages = 64,

	/* Values 65 -100 unused to avoid conflict with InterBase */
	
	frb_info_att_charset = 101,
	isc_info_db_class = 102,
	isc_info_firebird_version = 103,
	isc_info_oldest_transaction = 104,
	isc_info_oldest_active = 105,
	isc_info_oldest_snapshot = 106,
	isc_info_next_transaction = 107,
	isc_info_db_provider = 108,
	isc_info_active_transactions = 109,
	isc_info_active_tran_count = 110,
	isc_info_creation_date = 111,

	isc_info_db_last_value, // Aliasを除き、一番最後に配置すること

	// alias
	isc_info_version = isc_info_isc_version
};


/**************************************/
/* Database information return values */
/**************************************/

enum info_db_implementations
{
	isc_info_db_impl_rdb_vms = 1,
	isc_info_db_impl_rdb_eln = 2,
	isc_info_db_impl_rdb_eln_dev = 3,
	isc_info_db_impl_rdb_vms_y = 4,
	isc_info_db_impl_rdb_eln_y = 5,
	isc_info_db_impl_jri = 6,
	isc_info_db_impl_jsv = 7,

	isc_info_db_impl_isc_apl_68K = 25,
	isc_info_db_impl_isc_vax_ultr = 26,
	isc_info_db_impl_isc_vms = 27,
	isc_info_db_impl_isc_sun_68k = 28,
	isc_info_db_impl_isc_os2 = 29,
	isc_info_db_impl_isc_sun4 = 30,    /* 30 */
	
	isc_info_db_impl_isc_hp_ux = 31,
	isc_info_db_impl_isc_sun_386i = 32,
	isc_info_db_impl_isc_vms_orcl = 33,
	isc_info_db_impl_isc_mac_aux = 34,
	isc_info_db_impl_isc_rt_aix = 35,
	isc_info_db_impl_isc_mips_ult = 36,
	isc_info_db_impl_isc_xenix = 37,
	isc_info_db_impl_isc_dg = 38,
	isc_info_db_impl_isc_hp_mpexl = 39,
	isc_info_db_impl_isc_hp_ux68K = 40,   /* 40 */

	isc_info_db_impl_isc_sgi = 41,
	isc_info_db_impl_isc_sco_unix = 42,
	isc_info_db_impl_isc_cray = 43,
	isc_info_db_impl_isc_imp = 44,
	isc_info_db_impl_isc_delta = 45,
	isc_info_db_impl_isc_next = 46,
	isc_info_db_impl_isc_dos = 47,
	isc_info_db_impl_m88K = 48,
	isc_info_db_impl_unixware = 49,
	isc_info_db_impl_isc_winnt_x86 = 50,

	isc_info_db_impl_isc_epson = 51,
	isc_info_db_impl_alpha_osf = 52,
	isc_info_db_impl_alpha_vms = 53,
	isc_info_db_impl_netware_386 = 54, 
	isc_info_db_impl_win_only = 55,
	isc_info_db_impl_ncr_3000 = 56,
	isc_info_db_impl_winnt_ppc = 57,
	isc_info_db_impl_dg_x86 = 58,
	isc_info_db_impl_sco_ev = 59,
	isc_info_db_impl_i386 = 60,

	isc_info_db_impl_freebsd = 61,
	isc_info_db_impl_netbsd = 62,
	isc_info_db_impl_darwin = 63,
	isc_info_db_impl_sinixz = 64,

	isc_info_db_impl_linux_sparc = 65,
	isc_info_db_impl_linux_amd64 = 66,
	
	isc_info_db_impl_freebsd_amd64 = 67,

	isc_info_db_impl_winnt_amd64 = 68,
	isc_info_db_impl_linux_ppc = 69,

	isc_info_db_impl_last_value, // Aliasを除き、一番最後に配置すること

	// alias
	isc_info_db_impl_isc_a = isc_info_db_impl_isc_apl_68K,
	isc_info_db_impl_isc_u = isc_info_db_impl_isc_vax_ultr,
	isc_info_db_impl_isc_v = isc_info_db_impl_isc_vms,
	isc_info_db_impl_isc_s = isc_info_db_impl_isc_sun_68k,
};

enum info_db_class
{
	isc_info_db_class_access = 1,
	isc_info_db_class_y_valve = 2,
	isc_info_db_class_rem_int = 3,
	isc_info_db_class_rem_srvr = 4,
	isc_info_db_class_pipe_int = 7,
	isc_info_db_class_pipe_srvr = 8,
	isc_info_db_class_sam_int = 9,
	isc_info_db_class_sam_srvr = 10,
	isc_info_db_class_gateway = 11,
	isc_info_db_class_cache = 12,
	isc_info_db_class_classic_access = 13,
	isc_info_db_class_server_access = 14,

	isc_info_db_class_last_value   /* Leave this LAST! */
};

enum info_db_provider
{
	isc_info_db_code_rdb_eln = 1,
	isc_info_db_code_rdb_vms = 2,
	isc_info_db_code_interbase = 3,
	isc_info_db_code_firebird = 4,

	isc_info_db_code_last_value   /* Leave this LAST! */
};


/*****************************/
/* Request information items */
/*****************************/

immutable int isc_info_number_messages = 4;
immutable int isc_info_max_message = 5;
immutable int isc_info_max_send = 6;
immutable int isc_info_max_receive = 7;
immutable int isc_info_state = 8;
immutable int isc_info_message_number = 9;
immutable int isc_info_message_size = 10;
immutable int isc_info_request_cost = 11;
immutable int isc_info_access_path = 12;
immutable int isc_info_req_select_count = 13;
immutable int isc_info_req_insert_count = 14;
immutable int isc_info_req_update_count = 15;
immutable int isc_info_req_delete_count = 16;


/*********************/
/* Access path items */
/*********************/

immutable int isc_info_rsb_end = 0;
immutable int isc_info_rsb_begin = 1;
immutable int isc_info_rsb_type = 2;
immutable int isc_info_rsb_relation = 3;
immutable int isc_info_rsb_plan = 4;

/*************/
/* RecordSource (RSB) types */
/*************/

immutable int isc_info_rsb_unknown = 1;
immutable int isc_info_rsb_indexed = 2;
immutable int isc_info_rsb_navigate = 3;
immutable int isc_info_rsb_sequential = 4;
immutable int isc_info_rsb_cross = 5;
immutable int isc_info_rsb_sort = 6;
immutable int isc_info_rsb_first = 7;
immutable int isc_info_rsb_boolean = 8;
immutable int isc_info_rsb_union = 9;
immutable int isc_info_rsb_aggregate = 10;
immutable int isc_info_rsb_merge = 11;
immutable int isc_info_rsb_ext_sequential = 12;
immutable int isc_info_rsb_ext_indexed = 13;
immutable int isc_info_rsb_ext_dbkey = 14;
immutable int isc_info_rsb_left_cross = 15;
immutable int isc_info_rsb_select = 16;
immutable int isc_info_rsb_sql_join = 17;
immutable int isc_info_rsb_simulate = 18;
immutable int isc_info_rsb_sim_cross = 19;
immutable int isc_info_rsb_once = 20;
immutable int isc_info_rsb_procedure = 21;
immutable int isc_info_rsb_skip = 22;

/**********************/
/* Bitmap expressions */
/**********************/

immutable int isc_info_rsb_and = 1;
immutable int isc_info_rsb_or = 2;
immutable int isc_info_rsb_dbkey = 3;
immutable int isc_info_rsb_index = 4;

immutable int isc_info_req_active = 2;
immutable int isc_info_req_inactive = 3;
immutable int isc_info_req_send = 4;
immutable int isc_info_req_receive = 5;
immutable int isc_info_req_select = 6;
immutable int isc_info_req_sql_stall = 7;

/**************************/
/* Blob information items */
/**************************/

immutable int isc_info_blob_num_segments = 4;
immutable int isc_info_blob_max_segment = 5;
immutable int isc_info_blob_total_length = 6;
immutable int isc_info_blob_type = 7;

/*********************************/
/* Transaction information items */
/*********************************/

immutable int isc_info_tra_id = 4;
immutable int isc_info_tra_oldest_interesting = 5;
immutable int isc_info_tra_oldest_snapshot = 6;
immutable int isc_info_tra_oldest_active = 7;
immutable int isc_info_tra_isolation = 8;
immutable int isc_info_tra_access = 9;
immutable int isc_info_tra_lock_timeout = 10;

// isc_info_tra_isolation responses
immutable int isc_info_tra_consistency = 1;
immutable int isc_info_tra_concurrency = 2;
immutable int isc_info_tra_read_committed = 3;

// isc_info_tra_read_committed options
immutable int isc_info_tra_no_rec_version = 0;
immutable int isc_info_tra_rec_version = 1;

// isc_info_tra_access responses
immutable int isc_info_tra_readonly = 0;
immutable int isc_info_tra_readwrite = 1;


/*************************/
/* SQL information items */
/*************************/

immutable int isc_info_sql_select = 4;
immutable int isc_info_sql_bind = 5;
immutable int isc_info_sql_num_variables = 6;
immutable int isc_info_sql_describe_vars = 7;
immutable int isc_info_sql_describe_end = 8;
immutable int isc_info_sql_sqlda_seq = 9;
immutable int isc_info_sql_message_seq = 10;
immutable int isc_info_sql_type = 11;
immutable int isc_info_sql_sub_type = 12;
immutable int isc_info_sql_scale = 13;
immutable int isc_info_sql_length = 14;
immutable int isc_info_sql_null_ind = 15;
immutable int isc_info_sql_field = 16;
immutable int isc_info_sql_relation = 17;
immutable int isc_info_sql_owner = 18;
immutable int isc_info_sql_alias = 19;
immutable int isc_info_sql_sqlda_start = 20;
immutable int isc_info_sql_stmt_type = 21;
immutable int isc_info_sql_get_plan = 22;
immutable int isc_info_sql_records = 23;
immutable int isc_info_sql_batch_fetch = 24;
immutable int isc_info_sql_relation_alias = 25;

/*********************************/
/* SQL information return values */
/*********************************/

immutable int isc_info_sql_stmt_select = 1;
immutable int isc_info_sql_stmt_insert = 2;
immutable int isc_info_sql_stmt_update = 3;
immutable int isc_info_sql_stmt_delete = 4;
immutable int isc_info_sql_stmt_ddl = 5;
immutable int isc_info_sql_stmt_get_segment = 6;
immutable int isc_info_sql_stmt_put_segment = 7;
immutable int isc_info_sql_stmt_exec_procedure = 8;
immutable int isc_info_sql_stmt_start_trans = 9;
immutable int isc_info_sql_stmt_commit = 10;
immutable int isc_info_sql_stmt_rollback = 11;
immutable int isc_info_sql_stmt_select_for_upd = 12;
immutable int isc_info_sql_stmt_set_generator = 13;
immutable int isc_info_sql_stmt_savepoint = 14;



/*****************************
 * Service action items      *
 *****************************/

immutable int isc_action_svc_backup = 1; /* Starts database backup process on the server */
immutable int isc_action_svc_restore = 2; /* Starts database restore process on the server */
immutable int isc_action_svc_repair = 3; /* Starts database repair process on the server */
immutable int isc_action_svc_add_user = 4; /* Adds a new user to the security database */
immutable int isc_action_svc_delete_user = 5; /* Deletes a user record from the security database */
immutable int isc_action_svc_modify_user = 6; /* Modifies a user record in the security database */
immutable int isc_action_svc_display_user = 7; /* Displays a user record from the security database */
immutable int isc_action_svc_properties = 8; /* Sets database properties */
immutable int isc_action_svc_add_license = 9; /* Adds a license to the license file */
immutable int isc_action_svc_remove_license = 10; /* Removes a license from the license file */
immutable int isc_action_svc_db_stats = 11; /* Retrieves database statistics */
immutable int isc_action_svc_get_ib_log = 12; /* Retrieves the InterBase log file from the server */
immutable int isc_action_svc_get_fb_log = 12; /* Retrieves the Firebird log file from the server */

/*****************************
 * Service information items *
 *****************************/

immutable int isc_info_svc_svr_db_info = 50; /* Retrieves the number of attachments and databases */
immutable int isc_info_svc_get_license = 51; /* Retrieves all license keys and IDs from the license file */
immutable int isc_info_svc_get_license_mask = 52; /* Retrieves a bitmask representing licensed options on the server */
immutable int isc_info_svc_get_config = 53; /* Retrieves the parameters and values for IB_CONFIG */
immutable int isc_info_svc_version = 54; /* Retrieves the version of the services manager */
immutable int isc_info_svc_server_version = 55; /* Retrieves the version of the InterBase server */
immutable int isc_info_svc_implementation = 56; /* Retrieves the implementation of the InterBase server */
immutable int isc_info_svc_capabilities = 57; /* Retrieves a bitmask representing the server's capabilities */
immutable int isc_info_svc_user_dbpath = 58; /* Retrieves the path to the security database in use by the server */
immutable int isc_info_svc_get_env = 59; /* Retrieves the setting of $INTERBASE */
immutable int isc_info_svc_get_env_lock = 60; /* Retrieves the setting of $INTERBASE_LCK */
immutable int isc_info_svc_get_env_msg = 61; /* Retrieves the setting of $INTERBASE_MSG */
immutable int isc_info_svc_line = 62; /* Retrieves 1 line of service output per call */
immutable int isc_info_svc_to_eof = 63; /* Retrieves as much of the server output as will fit in the supplied buffer */
immutable int isc_info_svc_timeout = 64; /* Sets / signifies a timeout value for reading service information */
immutable int isc_info_svc_get_licensed_users = 65; /* Retrieves the number of users licensed for accessing the server */
immutable int isc_info_svc_limbo_trans = 66; /* Retrieve the limbo transactions */
immutable int isc_info_svc_running = 67; /* Checks to see if a service is running on an attachment */
immutable int isc_info_svc_get_users = 68; /* Returns the user information from isc_action_svc_display_users */

/******************************************************
 * Parameters for isc_action_{add|del|mod|disp)_user  *
 ******************************************************/

immutable int isc_spb_sec_userid = 5;
immutable int isc_spb_sec_groupid = 6;
immutable int isc_spb_sec_username = 7;
immutable int isc_spb_sec_password = 8;
immutable int isc_spb_sec_groupname = 9;
immutable int isc_spb_sec_firstname = 10;
immutable int isc_spb_sec_middlename = 11;
immutable int isc_spb_sec_lastname = 12;

/*******************************************************
 * Parameters for isc_action_svc_(add|remove)_license, *
 * isc_info_svc_get_license                            *
 *******************************************************/

immutable int isc_spb_lic_key = 5;
immutable int isc_spb_lic_id = 6;
immutable int isc_spb_lic_desc = 7;


/*****************************************
 * Parameters for isc_action_svc_backup  *
 *****************************************/

immutable int isc_spb_bkp_file = 5;
immutable int isc_spb_bkp_factor = 6;
immutable int isc_spb_bkp_length = 7;
immutable int isc_spb_bkp_ignore_checksums = 0x01;
immutable int isc_spb_bkp_ignore_limbo = 0x02;
immutable int isc_spb_bkp_metadata_only = 0x04;
immutable int isc_spb_bkp_no_garbage_collect = 0x08;
immutable int isc_spb_bkp_old_descriptions = 0x10;
immutable int isc_spb_bkp_non_transportable = 0x20;
immutable int isc_spb_bkp_convert = 0x40;
immutable int isc_spb_bkp_expand = 0x80;

/********************************************
 * Parameters for isc_action_svc_properties *
 ********************************************/

immutable int isc_spb_prp_page_buffers = 5;
immutable int isc_spb_prp_sweep_interval = 6;
immutable int isc_spb_prp_shutdown_db = 7;
immutable int isc_spb_prp_deny_new_attachments = 9;
immutable int isc_spb_prp_deny_new_transactions = 10;
immutable int isc_spb_prp_reserve_space = 11;
immutable int isc_spb_prp_write_mode = 12;
immutable int isc_spb_prp_access_mode = 13;
immutable int isc_spb_prp_set_sql_dialect = 14;
immutable int isc_spb_prp_activate = 0x0100;
immutable int isc_spb_prp_db_online = 0x0200;

/********************************************
 * Parameters for isc_spb_prp_reserve_space *
 ********************************************/

immutable int isc_spb_prp_res_use_full = 35;
immutable int isc_spb_prp_res = 36;

/******************************************
 * Parameters for isc_spb_prp_write_mode  *
 ******************************************/

immutable int isc_spb_prp_wm_async = 37;
immutable int isc_spb_prp_wm_sync = 38;

/******************************************
 * Parameters for isc_spb_prp_access_mode *
 ******************************************/

immutable int isc_spb_prp_am_readonly = 39;
immutable int isc_spb_prp_am_readwrite = 40;

/*****************************************
 * Parameters for isc_action_svc_repair  *
 *****************************************/

immutable int isc_spb_rpr_commit_trans = 15;
immutable int isc_spb_rpr_rollback_trans = 34;
immutable int isc_spb_rpr_recover_two_phase = 17;
immutable int isc_spb_tra_id = 18;
immutable int isc_spb_single_tra_id = 19;
immutable int isc_spb_multi_tra_id = 20;
immutable int isc_spb_tra_state = 21;
immutable int isc_spb_tra_state_limbo = 22;
immutable int isc_spb_tra_state_commit = 23;
immutable int isc_spb_tra_state_rollback = 24;
immutable int isc_spb_tra_state_unknown = 25;
immutable int isc_spb_tra_host_site = 26;
immutable int isc_spb_tra_remote_site = 27;
immutable int isc_spb_tra_db_path = 28;
immutable int isc_spb_tra_advise = 29;
immutable int isc_spb_tra_advise_commit = 30;
immutable int isc_spb_tra_advise_rollback = 31;
immutable int isc_spb_tra_advise_unknown = 33;

immutable int isc_spb_rpr_validate_db = 0x01;
immutable int isc_spb_rpr_sweep_db = 0x02;
immutable int isc_spb_rpr_mend_db = 0x04;
immutable int isc_spb_rpr_list_limbo_trans = 0x08;
immutable int isc_spb_rpr_check_db = 0x10;
immutable int isc_spb_rpr_ignore_checksum = 0x20;
immutable int isc_spb_rpr_kill_shadows = 0x40;
immutable int isc_spb_rpr_full = 0x80;

/*****************************************
 * Parameters for isc_action_svc_restore *
 *****************************************/

immutable int isc_spb_res_buffers = 9;
immutable int isc_spb_res_page_size = 10;
immutable int isc_spb_res_length = 11;
immutable int isc_spb_res_access_mode = 12;
immutable int isc_spb_res_deactivate_idx = 0x0100;
immutable int isc_spb_res_no_shadow = 0x0200;
immutable int isc_spb_res_no_validity = 0x0400;
immutable int isc_spb_res_one_at_a_time = 0x0800;
immutable int isc_spb_res_replace = 0x1000;
immutable int isc_spb_res_create = 0x2000;
immutable int isc_spb_res_use_all_space = 0x4000;

/******************************************
 * Parameters for isc_spb_res_access_mode  *
 ******************************************/

immutable int isc_spb_res_am_readonly = isc_spb_prp_am_readonly;
immutable int isc_spb_res_am_readwrite = isc_spb_prp_am_readwrite;

/*******************************************
 * Parameters for isc_info_svc_svr_db_info *
 *******************************************/

immutable int isc_spb_num_att = 5;
immutable int isc_spb_num_db = 6;

/*****************************************
 * Parameters for isc_info_svc_db_stats  *
 *****************************************/

immutable int isc_spb_sts_data_pages = 0x01;
immutable int isc_spb_sts_db_log = 0x02;
immutable int isc_spb_sts_hdr_pages = 0x04;
immutable int isc_spb_sts_idx_pages = 0x08;
immutable int isc_spb_sts_sys_relations = 0x10;
immutable int isc_spb_sts_record_versions = 0x20;
immutable int isc_spb_sts_table = 0x40;
immutable int isc_spb_sts_nocreation = 0x80;

/***********************************/
/* Server configuration key values */
/***********************************/

/* Not available in Firebird 1.5 */


/**********************************************/
/* Dynamic Data Definition Language operators */
/**********************************************/

/******************/
/* Version number */
/******************/

immutable int isc_dyn_version_1 = 1;
immutable int isc_dyn_eoc = 255;

/******************************/
/* Operations (may be nested) */
/******************************/

immutable int isc_dyn_begin = 2;
immutable int isc_dyn_end = 3;
immutable int isc_dyn_if = 4;
immutable int isc_dyn_def_database = 5;
immutable int isc_dyn_def_global_fld = 6;
immutable int isc_dyn_def_local_fld = 7;
immutable int isc_dyn_def_idx = 8;
immutable int isc_dyn_def_rel = 9;
immutable int isc_dyn_def_sql_fld = 10;
immutable int isc_dyn_def_view = 12;
immutable int isc_dyn_def_trigger = 15;
immutable int isc_dyn_def_security_class = 120;
immutable int isc_dyn_def_dimension = 140;
immutable int isc_dyn_def_generator = 24;
immutable int isc_dyn_def_function = 25;
immutable int isc_dyn_def_filter = 26;
immutable int isc_dyn_def_function_arg = 27;
immutable int isc_dyn_def_shadow = 34;
immutable int isc_dyn_def_trigger_msg = 17;
immutable int isc_dyn_def_file = 36;
immutable int isc_dyn_mod_database = 39;
immutable int isc_dyn_mod_rel = 11;
immutable int isc_dyn_mod_global_fld = 13;
immutable int isc_dyn_mod_idx = 102;
immutable int isc_dyn_mod_local_fld = 14;
immutable int isc_dyn_mod_sql_fld = 216;
immutable int isc_dyn_mod_view = 16;
immutable int isc_dyn_mod_security_class = 122;
immutable int isc_dyn_mod_trigger = 113;
immutable int isc_dyn_mod_trigger_msg = 28;
immutable int isc_dyn_delete_database = 18;
immutable int isc_dyn_delete_rel = 19;
immutable int isc_dyn_delete_global_fld = 20;
immutable int isc_dyn_delete_local_fld = 21;
immutable int isc_dyn_delete_idx = 22;
immutable int isc_dyn_delete_security_class = 123;
immutable int isc_dyn_delete_dimensions = 143;
immutable int isc_dyn_delete_trigger = 23;
immutable int isc_dyn_delete_trigger_msg = 29;
immutable int isc_dyn_delete_filter = 32;
immutable int isc_dyn_delete_function = 33;
immutable int isc_dyn_delete_shadow = 35;
immutable int isc_dyn_grant = 30;
immutable int isc_dyn_revoke = 31;
immutable int isc_dyn_def_primary_key = 37;
immutable int isc_dyn_def_foreign_key = 38;
immutable int isc_dyn_def_unique = 40;
immutable int isc_dyn_def_procedure = 164;
immutable int isc_dyn_delete_procedure = 165;
immutable int isc_dyn_def_parameter = 135;
immutable int isc_dyn_delete_parameter = 136;
immutable int isc_dyn_mod_procedure = 175;
/* Deprecated.
immutable int isc_dyn_def_log_file = 176;
immutable int isc_dyn_def_cache_file = 180;
*/
immutable int isc_dyn_def_exception = 181;
immutable int isc_dyn_mod_exception = 182;
immutable int isc_dyn_del_exception = 183;
/* Deprecated.
immutable int isc_dyn_drop_log = 194;
immutable int isc_dyn_drop_cache = 195;
immutable int isc_dyn_def_default_log = 202;
*/
immutable int isc_dyn_def_difference = 220;
immutable int isc_dyn_drop_difference = 221;
immutable int isc_dyn_begin_backup = 222;
immutable int isc_dyn_end_backup = 223;

/***********************/
/* View specific stuff */
/***********************/

immutable int isc_dyn_view_blr = 43;
immutable int isc_dyn_view_source = 44;
immutable int isc_dyn_view_relation = 45;
immutable int isc_dyn_view_context = 46;
immutable int isc_dyn_view_context_name = 47;

/**********************/
/* Generic attributes */
/**********************/

immutable int isc_dyn_rel_name = 50;
immutable int isc_dyn_fld_name = 51;
immutable int isc_dyn_new_fld_name = 215;
immutable int isc_dyn_idx_name = 52;
immutable int isc_dyn_description = 53;
immutable int isc_dyn_security_class = 54;
immutable int isc_dyn_system_flag = 55;
immutable int isc_dyn_update_flag = 56;
immutable int isc_dyn_prc_name = 166;
immutable int isc_dyn_prm_name = 137;
immutable int isc_dyn_sql_object = 196;
immutable int isc_dyn_fld_character_set_name = 174;

/********************************/
/* Relation specific attributes */
/********************************/

immutable int isc_dyn_rel_dbkey_length = 61;
immutable int isc_dyn_rel_store_trig = 62;
immutable int isc_dyn_rel_modify_trig = 63;
immutable int isc_dyn_rel_erase_trig = 64;
immutable int isc_dyn_rel_store_trig_source = 65;
immutable int isc_dyn_rel_modify_trig_source = 66;
immutable int isc_dyn_rel_erase_trig_source = 67;
immutable int isc_dyn_rel_ext_file = 68;
immutable int isc_dyn_rel_sql_protection = 69;
immutable int isc_dyn_rel_constraint = 162;
immutable int isc_dyn_delete_rel_constraint = 163;

/************************************/
/* Global field specific attributes */
/************************************/

immutable int isc_dyn_fld_type = 70;
immutable int isc_dyn_fld_length = 71;
immutable int isc_dyn_fld_scale = 72;
immutable int isc_dyn_fld_sub_type = 73;
immutable int isc_dyn_fld_segment_length = 74;
immutable int isc_dyn_fld_query_header = 75;
immutable int isc_dyn_fld_edit_string = 76;
immutable int isc_dyn_fld_validation_blr = 77;
immutable int isc_dyn_fld_validation_source = 78;
immutable int isc_dyn_fld_computed_blr = 79;
immutable int isc_dyn_fld_computed_source = 80;
immutable int isc_dyn_fld_missing_value = 81;
immutable int isc_dyn_fld_default_value = 82;
immutable int isc_dyn_fld_query_name = 83;
immutable int isc_dyn_fld_dimensions = 84;
immutable int isc_dyn_fld_not_null = 85;
immutable int isc_dyn_fld_precision = 86;
immutable int isc_dyn_fld_char_length = 172;
immutable int isc_dyn_fld_collation = 173;
immutable int isc_dyn_fld_default_source = 193;
immutable int isc_dyn_del_default = 197;
immutable int isc_dyn_del_validation = 198;
immutable int isc_dyn_single_validation = 199;
immutable int isc_dyn_fld_character_set = 203;

/***********************************/
/* Local field specific attributes */
/***********************************/

immutable int isc_dyn_fld_source = 90;
immutable int isc_dyn_fld_base_fld = 91;
immutable int isc_dyn_fld_position = 92;
immutable int isc_dyn_fld_update_flag = 93;

/*****************************/
/* Index specific attributes */
/*****************************/

immutable int isc_dyn_idx_unique = 100;
immutable int isc_dyn_idx_inactive = 101;
immutable int isc_dyn_idx_type = 103;
immutable int isc_dyn_idx_foreign_key = 104;
immutable int isc_dyn_idx_ref_column = 105;
immutable int isc_dyn_idx_statistic = 204;

/*******************************/
/* Trigger specific attributes */
/*******************************/

immutable int isc_dyn_trg_type = 110;
immutable int isc_dyn_trg_blr = 111;
immutable int isc_dyn_trg_source = 112;
immutable int isc_dyn_trg_name = 114;
immutable int isc_dyn_trg_sequence = 115;
immutable int isc_dyn_trg_inactive = 116;
immutable int isc_dyn_trg_msg_number = 117;
immutable int isc_dyn_trg_msg = 118;

/**************************************/
/* Security Class specific attributes */
/**************************************/

immutable int isc_dyn_scl_acl = 121;
immutable int isc_dyn_grant_user = 130;
immutable int isc_dyn_grant_user_explicit = 219;
immutable int isc_dyn_grant_proc = 186;
immutable int isc_dyn_grant_trig = 187;
immutable int isc_dyn_grant_view = 188;
immutable int isc_dyn_grant_options = 132;
immutable int isc_dyn_grant_user_group = 205;
immutable int isc_dyn_grant_role = 218;


/**********************************/
/* Dimension specific information */
/**********************************/

immutable int isc_dyn_dim_lower = 141;
immutable int isc_dyn_dim_upper = 142;

/****************************/
/* File specific attributes */
/****************************/

immutable int isc_dyn_file_name = 125;
immutable int isc_dyn_file_start = 126;
immutable int isc_dyn_file_length = 127;
immutable int isc_dyn_shadow_number = 128;
immutable int isc_dyn_shadow_man_auto = 129;
immutable int isc_dyn_shadow_conditional = 130;

/********************************/
/* Log file specific attributes */
/********************************/
/* Deprecated.
immutable int isc_dyn_log_file_sequence = 177;
immutable int isc_dyn_log_file_partitions = 178;
immutable int isc_dyn_log_file_serial = 179;
immutable int isc_dyn_log_file_overflow = 200;
immutable int isc_dyn_log_file_raw = 201;
*/

/***************************/
/* Log specific attributes */
/***************************/
/* Deprecated.
immutable int isc_dyn_log_group_commit_wait = 189;
immutable int isc_dyn_log_buffer_size = 190;
immutable int isc_dyn_log_check_point_length = 191;
immutable int isc_dyn_log_num_of_buffers = 192;
*/

/********************************/
/* Function specific attributes */
/********************************/

immutable int isc_dyn_function_name = 145;
immutable int isc_dyn_function_type = 146;
immutable int isc_dyn_func_module_name = 147;
immutable int isc_dyn_func_entry_point = 148;
immutable int isc_dyn_func_return_argument = 149;
immutable int isc_dyn_func_arg_position = 150;
immutable int isc_dyn_func_mechanism = 151;
immutable int isc_dyn_filter_in_subtype = 152;
immutable int isc_dyn_filter_out_subtype = 153;


immutable int isc_dyn_description2 = 154;
immutable int isc_dyn_fld_computed_source2 = 155;
immutable int isc_dyn_fld_edit_string2 = 156;
immutable int isc_dyn_fld_query_header2 = 157;
immutable int isc_dyn_fld_validation_source2 = 158;
immutable int isc_dyn_trg_msg2 = 159;
immutable int isc_dyn_trg_source2 = 160;
immutable int isc_dyn_view_source2 = 161;
immutable int isc_dyn_xcp_msg2 = 184;

/*********************************/
/* Generator specific attributes */
/*********************************/

immutable int isc_dyn_generator_name = 95;
immutable int isc_dyn_generator_id = 96;

/*********************************/
/* Procedure specific attributes */
/*********************************/

immutable int isc_dyn_prc_inputs = 167;
immutable int isc_dyn_prc_outputs = 168;
immutable int isc_dyn_prc_source = 169;
immutable int isc_dyn_prc_blr = 170;
immutable int isc_dyn_prc_source2 = 171;

/*********************************/
/* Parameter specific attributes */
/*********************************/

immutable int isc_dyn_prm_number = 138;
immutable int isc_dyn_prm_type = 139;

/********************************/
/* Relation specific attributes */
/********************************/

immutable int isc_dyn_xcp_msg = 185;

/**********************************************/
/* Cascading referential integrity values     */
/**********************************************/
immutable int isc_dyn_foreign_key_update = 205;
immutable int isc_dyn_foreign_key_delete = 206;
immutable int isc_dyn_foreign_key_cascade = 207;
immutable int isc_dyn_foreign_key_default = 208;
immutable int isc_dyn_foreign_key_null = 209;
immutable int isc_dyn_foreign_key_none = 210;

/***********************/
/* SQL role values     */
/***********************/
immutable int isc_dyn_def_sql_role = 211;
immutable int isc_dyn_sql_role_name = 212;
immutable int isc_dyn_grant_admin_options = 213;
immutable int isc_dyn_del_sql_role = 214;
/* 215 & 216 are used some lines above. */

/**********************************************/
/* Generators again                           */
/**********************************************/

immutable int isc_dyn_delete_generator = 217;

// New for comments in objects.
immutable int isc_dyn_mod_function = 224;
immutable int isc_dyn_mod_filter = 225;
immutable int isc_dyn_mod_generator = 226;
immutable int isc_dyn_mod_sql_role = 227;
immutable int isc_dyn_mod_charset = 228;
immutable int isc_dyn_mod_collation = 229;
immutable int isc_dyn_mod_prc_parameter = 230;

/***********************/
/* collation values     */
/***********************/
immutable int isc_dyn_def_collation = 231;
immutable int isc_dyn_coll_for_charset = 232;
immutable int isc_dyn_coll_from = 233;
immutable int isc_dyn_coll_attribute = 234;
immutable int isc_dyn_coll_specific_attributes_charset = 235;
immutable int isc_dyn_coll_specific_attributes = 236;
immutable int isc_dyn_del_collation = 237;

/****************************/
/* Last $dyn value assigned */
/****************************/

immutable int isc_dyn_last_dyn_value = 237;

/******************************************/
/* Array slice description language (SDL) */
/******************************************/

immutable int isc_sdl_version1 = 1;
immutable int isc_sdl_eoc = 255;
immutable int isc_sdl_relation = 2;
immutable int isc_sdl_rid = 3;
immutable int isc_sdl_field = 4;
immutable int isc_sdl_fid = 5;
immutable int isc_sdl_struct = 6;
immutable int isc_sdl_variable = 7;
immutable int isc_sdl_scalar = 8;
immutable int isc_sdl_tiny_integer = 9;
immutable int isc_sdl_short_integer = 10;
immutable int isc_sdl_long_integer = 11;
immutable int isc_sdl_literal = 12;
immutable int isc_sdl_add = 13;
immutable int isc_sdl_subtract = 14;
immutable int isc_sdl_multiply = 15;
immutable int isc_sdl_divide = 16;
immutable int isc_sdl_negate = 17;
immutable int isc_sdl_eql = 18;
immutable int isc_sdl_neq = 19;
immutable int isc_sdl_gtr = 20;
immutable int isc_sdl_geq = 21;
immutable int isc_sdl_lss = 22;
immutable int isc_sdl_leq = 23;
immutable int isc_sdl_and = 24;
immutable int isc_sdl_or = 25;
immutable int isc_sdl_not = 26;
immutable int isc_sdl_while = 27;
immutable int isc_sdl_assignment = 28;
immutable int isc_sdl_label = 29;
immutable int isc_sdl_leave = 30;
immutable int isc_sdl_begin = 31;
immutable int isc_sdl_end = 32;
immutable int isc_sdl_do3 = 33;
immutable int isc_sdl_do2 = 34;
immutable int isc_sdl_do1 = 35;
immutable int isc_sdl_element = 36;

/********************************************/
/* International text interpretation values */
/********************************************/

immutable int isc_interp_eng_ascii = 0;
immutable int isc_interp_jpn_sjis = 5;
immutable int isc_interp_jpn_euc = 6;

/*****************/
/* Blob Subtypes */
/*****************/

/* types less than zero are reserved for customer use */

immutable int isc_blob_untyped = 0;

/* internal subtypes */

immutable int isc_blob_text = 1;
immutable int isc_blob_blr = 2;
immutable int isc_blob_acl = 3;
immutable int isc_blob_ranges = 4;
immutable int isc_blob_summary = 5;
immutable int isc_blob_format = 6;
immutable int isc_blob_tra = 7;
immutable int isc_blob_extfile = 8;
immutable int isc_blob_max_predefined_subtype = 9;

/* the range 20-30 is reserved for dBASE and Paradox types */

immutable int isc_blob_formatted_memo = 20;
immutable int isc_blob_paradox_ole = 21;
immutable int isc_blob_graphic = 22;
immutable int isc_blob_dbase_ole = 23;
immutable int isc_blob_typed_binary = 24;

/* Deprecated definitions maintained for compatibility only */

immutable int isc_info_db_SQL_dialect = 62;
immutable int isc_dpb_SQL_dialect = 63;
immutable int isc_dpb_set_db_SQL_dialect = 65;


/***********************/
/*   ISC Error Codes   */
/***********************/


immutable int isc_facility = 20;
immutable int isc_base = 335544320;
immutable int isc_factor = 1;
immutable int isc_arg_end   = 0; // end of argument list 
immutable int isc_arg_gds   = 1; // generic DSRI status value 
immutable int isc_arg_string  = 2; // string argument 
immutable int isc_arg_cstring  = 3; // count & string argument 
immutable int isc_arg_number  = 4; // numeric argument (long) 
immutable int isc_arg_interpreted = 5; // interpreted status code (string) 
immutable int isc_arg_vms   = 6; // VAX/VMS status code (long) 
immutable int isc_arg_unix   = 7; // UNIX error code 
immutable int isc_arg_domain  = 8; // Apollo/Domain error code 
immutable int isc_arg_dos   = 9; // MSDOS/OS2 error code 
immutable int isc_arg_mpexl  = 10; // HP MPE/XL error code 
immutable int isc_arg_mpexl_ipc = 11; // HP MPE/XL IPC error code 
immutable int isc_arg_next_mach = 15; // NeXT/Mach error code 
immutable int isc_arg_netware  = 16; // NetWare error code 
immutable int isc_arg_win32  = 17; // Win32 error code 
immutable int isc_arg_warning  = 18; // warning argument 

immutable int isc_arith_except                     = 335544321;
immutable int isc_bad_dbkey                        = 335544322;
immutable int isc_bad_db_format                    = 335544323;
immutable int isc_bad_db_handle                    = 335544324;
immutable int isc_bad_dpb_content                  = 335544325;
immutable int isc_bad_dpb_form                     = 335544326;
immutable int isc_bad_req_handle                   = 335544327;
immutable int isc_bad_segstr_handle                = 335544328;
immutable int isc_bad_segstr_id                    = 335544329;
immutable int isc_bad_tpb_content                  = 335544330;
immutable int isc_bad_tpb_form                     = 335544331;
immutable int isc_bad_trans_handle                 = 335544332;
immutable int isc_bug_check                        = 335544333;
immutable int isc_convert_error                    = 335544334;
immutable int isc_db_corrupt                       = 335544335;
immutable int isc_deadlock                         = 335544336;
immutable int isc_excess_trans                     = 335544337;
immutable int isc_from_no_match                    = 335544338;
immutable int isc_infinap                          = 335544339;
immutable int isc_infona                           = 335544340;
immutable int isc_infunk                           = 335544341;
immutable int isc_integ_fail                       = 335544342;
immutable int isc_invalid_blr                      = 335544343;
immutable int isc_io_error                         = 335544344;
immutable int isc_lock_conflict                    = 335544345;
immutable int isc_metadata_corrupt                 = 335544346;
immutable int isc_not_valid                        = 335544347;
immutable int isc_no_cur_rec                       = 335544348;
immutable int isc_no_dup                           = 335544349;
immutable int isc_no_finish                        = 335544350;
immutable int isc_no_meta_update                   = 335544351;
immutable int isc_no_priv                          = 335544352;
immutable int isc_no_recon                         = 335544353;
immutable int isc_no_record                        = 335544354;
immutable int isc_no_segstr_close                  = 335544355;
immutable int isc_obsolete_metadata                = 335544356;
immutable int isc_open_trans                       = 335544357;
immutable int isc_port_len                         = 335544358;
immutable int isc_read_only_field                  = 335544359;
immutable int isc_read_only_rel                    = 335544360;
immutable int isc_read_only_trans                  = 335544361;
immutable int isc_read_only_view                   = 335544362;
immutable int isc_req_no_trans                     = 335544363;
immutable int isc_req_sync                         = 335544364;
immutable int isc_req_wrong_db                     = 335544365;
immutable int isc_segment                          = 335544366;
immutable int isc_segstr_eof                       = 335544367;
immutable int isc_segstr_no_op                     = 335544368;
immutable int isc_segstr_no_read                   = 335544369;
immutable int isc_segstr_no_trans                  = 335544370;
immutable int isc_segstr_no_write                  = 335544371;
immutable int isc_segstr_wrong_db                  = 335544372;
immutable int isc_sys_request                      = 335544373;
immutable int isc_stream_eof                       = 335544374;
immutable int isc_unavailable                      = 335544375;
immutable int isc_unres_rel                        = 335544376;
immutable int isc_uns_ext                          = 335544377;
immutable int isc_wish_list                        = 335544378;
immutable int isc_wrong_ods                        = 335544379;
immutable int isc_wronumarg                        = 335544380;
immutable int isc_imp_exc                          = 335544381;
immutable int isc_random                           = 335544382;
immutable int isc_fatal_conflict                   = 335544383;
immutable int isc_badblk                           = 335544384;
immutable int isc_invpoolcl                        = 335544385;
immutable int isc_nopoolids                        = 335544386;
immutable int isc_relbadblk                        = 335544387;
immutable int isc_blktoobig                        = 335544388;
immutable int isc_bufexh                           = 335544389;
immutable int isc_syntaxerr                        = 335544390;
immutable int isc_bufinuse                         = 335544391;
immutable int isc_bdbincon                         = 335544392;
immutable int isc_reqinuse                         = 335544393;
immutable int isc_badodsver                        = 335544394;
immutable int isc_relnotdef                        = 335544395;
immutable int isc_fldnotdef                        = 335544396;
immutable int isc_dirtypage                        = 335544397;
immutable int isc_waifortra                        = 335544398;
immutable int isc_doubleloc                        = 335544399;
immutable int isc_nodnotfnd                        = 335544400;
immutable int isc_dupnodfnd                        = 335544401;
immutable int isc_locnotmar                        = 335544402;
immutable int isc_badpagtyp                        = 335544403;
immutable int isc_corrupt                          = 335544404;
immutable int isc_badpage                          = 335544405;
immutable int isc_badindex                         = 335544406;
immutable int isc_dbbnotzer                        = 335544407;
immutable int isc_tranotzer                        = 335544408;
immutable int isc_trareqmis                        = 335544409;
immutable int isc_badhndcnt                        = 335544410;
immutable int isc_wrotpbver                        = 335544411;
immutable int isc_wroblrver                        = 335544412;
immutable int isc_wrodpbver                        = 335544413;
immutable int isc_blobnotsup                       = 335544414;
immutable int isc_badrelation                      = 335544415;
immutable int isc_nodetach                         = 335544416;
immutable int isc_notremote                        = 335544417;
immutable int isc_trainlim                         = 335544418;
immutable int isc_notinlim                         = 335544419;
immutable int isc_traoutsta                        = 335544420;
immutable int isc_connect_reject                   = 335544421;
immutable int isc_dbfile                           = 335544422;
immutable int isc_orphan                           = 335544423;
immutable int isc_no_lock_mgr                      = 335544424;
immutable int isc_ctxinuse                         = 335544425;
immutable int isc_ctxnotdef                        = 335544426;
immutable int isc_datnotsup                        = 335544427;
immutable int isc_badmsgnum                        = 335544428;
immutable int isc_badparnum                        = 335544429;
immutable int isc_virmemexh                        = 335544430;
immutable int isc_blocking_signal                  = 335544431;
immutable int isc_lockmanerr                       = 335544432;
immutable int isc_journerr                         = 335544433;
immutable int isc_keytoobig                        = 335544434;
immutable int isc_nullsegkey                       = 335544435;
immutable int isc_sqlerr                           = 335544436;
immutable int isc_wrodynver                        = 335544437;
immutable int isc_funnotdef                        = 335544438;
immutable int isc_funmismat                        = 335544439;
immutable int isc_bad_msg_vec                      = 335544440;
immutable int isc_bad_detach                       = 335544441;
immutable int isc_noargacc_read                    = 335544442;
immutable int isc_noargacc_write                   = 335544443;
immutable int isc_read_only                        = 335544444;
immutable int isc_ext_err                          = 335544445;
immutable int isc_non_updatable                    = 335544446;
immutable int isc_no_rollback                      = 335544447;
immutable int isc_bad_sec_info                     = 335544448;
immutable int isc_invalid_sec_info                 = 335544449;
immutable int isc_misc_interpreted                 = 335544450;
immutable int isc_update_conflict                  = 335544451;
immutable int isc_unlicensed                       = 335544452;
immutable int isc_obj_in_use                       = 335544453;
immutable int isc_nofilter                         = 335544454;
immutable int isc_shadow_accessed                  = 335544455;
immutable int isc_invalid_sdl                      = 335544456;
immutable int isc_out_of_bounds                    = 335544457;
immutable int isc_invalid_dimension                = 335544458;
immutable int isc_rec_in_limbo                     = 335544459;
immutable int isc_shadow_missing                   = 335544460;
immutable int isc_cant_validate                    = 335544461;
immutable int isc_cant_start_journal               = 335544462;
immutable int isc_gennotdef                        = 335544463;
immutable int isc_cant_start_logging               = 335544464;
immutable int isc_bad_segstr_type                  = 335544465;
immutable int isc_foreign_key                      = 335544466;
immutable int isc_high_minor                       = 335544467;
immutable int isc_tra_state                        = 335544468;
immutable int isc_trans_invalid                    = 335544469;
immutable int isc_buf_invalid                      = 335544470;
immutable int isc_indexnotdefined                  = 335544471;
immutable int isc_login                            = 335544472;
immutable int isc_invalid_bookmark                 = 335544473;
immutable int isc_bad_lock_level                   = 335544474;
immutable int isc_relation_lock                    = 335544475;
immutable int isc_record_lock                      = 335544476;
immutable int isc_max_idx                          = 335544477;
immutable int isc_jrn_enable                       = 335544478;
immutable int isc_old_failure                      = 335544479;
immutable int isc_old_in_progress                  = 335544480;
immutable int isc_old_no_space                     = 335544481;
immutable int isc_no_wal_no_jrn                    = 335544482;
immutable int isc_num_old_files                    = 335544483;
immutable int isc_wal_file_open                    = 335544484;
immutable int isc_bad_stmt_handle                  = 335544485;
immutable int isc_wal_failure                      = 335544486;
immutable int isc_walw_err                         = 335544487;
immutable int isc_logh_small                       = 335544488;
immutable int isc_logh_inv_version                 = 335544489;
immutable int isc_logh_open_flag                   = 335544490;
immutable int isc_logh_open_flag2                  = 335544491;
immutable int isc_logh_diff_dbname                 = 335544492;
immutable int isc_logf_unexpected_eof              = 335544493;
immutable int isc_logr_incomplete                  = 335544494;
immutable int isc_logr_header_small                = 335544495;
immutable int isc_logb_small                       = 335544496;
immutable int isc_wal_illegal_attach               = 335544497;
immutable int isc_wal_invalid_wpb                  = 335544498;
immutable int isc_wal_err_rollover                 = 335544499;
immutable int isc_no_wal                           = 335544500;
immutable int isc_drop_wal                         = 335544501;
immutable int isc_stream_not_defined               = 335544502;
immutable int isc_wal_subsys_error                 = 335544503;
immutable int isc_wal_subsys_corrupt               = 335544504;
immutable int isc_no_archive                       = 335544505;
immutable int isc_shutinprog                       = 335544506;
immutable int isc_range_in_use                     = 335544507;
immutable int isc_range_not_found                  = 335544508;
immutable int isc_charset_not_found                = 335544509;
immutable int isc_lock_timeout                     = 335544510;
immutable int isc_prcnotdef                        = 335544511;
immutable int isc_prcmismat                        = 335544512;
immutable int isc_wal_bugcheck                     = 335544513;
immutable int isc_wal_cant_expand                  = 335544514;
immutable int isc_codnotdef                        = 335544515;
immutable int isc_xcpnotdef                        = 335544516;
immutable int isc_except                           = 335544517;
immutable int isc_cache_restart                    = 335544518;
immutable int isc_bad_lock_handle                  = 335544519;
immutable int isc_jrn_present                      = 335544520;
immutable int isc_wal_err_rollover2                = 335544521;
immutable int isc_wal_err_logwrite                 = 335544522;
immutable int isc_wal_err_jrn_comm                 = 335544523;
immutable int isc_wal_err_expansion                = 335544524;
immutable int isc_wal_err_setup                    = 335544525;
immutable int isc_wal_err_ww_sync                  = 335544526;
immutable int isc_wal_err_ww_start                 = 335544527;
immutable int isc_shutdown                         = 335544528;
immutable int isc_existing_priv_mod                = 335544529;
immutable int isc_primary_key_ref                  = 335544530;
immutable int isc_primary_key_notnull              = 335544531;
immutable int isc_ref_cnstrnt_notfound             = 335544532;
immutable int isc_foreign_key_notfound             = 335544533;
immutable int isc_ref_cnstrnt_update               = 335544534;
immutable int isc_check_cnstrnt_update             = 335544535;
immutable int isc_check_cnstrnt_del                = 335544536;
immutable int isc_integ_index_seg_del              = 335544537;
immutable int isc_integ_index_seg_mod              = 335544538;
immutable int isc_integ_index_del                  = 335544539;
immutable int isc_integ_index_mod                  = 335544540;
immutable int isc_check_trig_del                   = 335544541;
immutable int isc_check_trig_update                = 335544542;
immutable int isc_cnstrnt_fld_del                  = 335544543;
immutable int isc_cnstrnt_fld_rename               = 335544544;
immutable int isc_rel_cnstrnt_update               = 335544545;
immutable int isc_constaint_on_view                = 335544546;
immutable int isc_invld_cnstrnt_type               = 335544547;
immutable int isc_primary_key_exists               = 335544548;
immutable int isc_systrig_update                   = 335544549;
immutable int isc_not_rel_owner                    = 335544550;
immutable int isc_grant_obj_notfound               = 335544551;
immutable int isc_grant_fld_notfound               = 335544552;
immutable int isc_grant_nopriv                     = 335544553;
immutable int isc_nonsql_security_rel              = 335544554;
immutable int isc_nonsql_security_fld              = 335544555;
immutable int isc_wal_cache_err                    = 335544556;
immutable int isc_shutfail                         = 335544557;
immutable int isc_check_constraint                 = 335544558;
immutable int isc_bad_svc_handle                   = 335544559;
immutable int isc_shutwarn                         = 335544560;
immutable int isc_wrospbver                        = 335544561;
immutable int isc_bad_spb_form                     = 335544562;
immutable int isc_svcnotdef                        = 335544563;
immutable int isc_no_jrn                           = 335544564;
immutable int isc_transliteration_failed           = 335544565;
immutable int isc_start_cm_for_wal                 = 335544566;
immutable int isc_wal_ovflow_log_required          = 335544567;
immutable int isc_text_subtype                     = 335544568;
immutable int isc_dsql_error                       = 335544569;
immutable int isc_dsql_command_err                 = 335544570;
immutable int isc_dsql_constant_err                = 335544571;
immutable int isc_dsql_cursor_err                  = 335544572;
immutable int isc_dsql_datatype_err                = 335544573;
immutable int isc_dsql_decl_err                    = 335544574;
immutable int isc_dsql_cursor_update_err           = 335544575;
immutable int isc_dsql_cursor_open_err             = 335544576;
immutable int isc_dsql_cursor_close_err            = 335544577;
immutable int isc_dsql_field_err                   = 335544578;
immutable int isc_dsql_internal_err                = 335544579;
immutable int isc_dsql_relation_err                = 335544580;
immutable int isc_dsql_procedure_err               = 335544581;
immutable int isc_dsql_request_err                 = 335544582;
immutable int isc_dsql_sqlda_err                   = 335544583;
immutable int isc_dsql_var_count_err               = 335544584;
immutable int isc_dsql_stmt_handle                 = 335544585;
immutable int isc_dsql_function_err                = 335544586;
immutable int isc_dsql_blob_err                    = 335544587;
immutable int isc_collation_not_found              = 335544588;
immutable int isc_collation_not_for_charset        = 335544589;
immutable int isc_dsql_dup_option                  = 335544590;
immutable int isc_dsql_tran_err                    = 335544591;
immutable int isc_dsql_invalid_array               = 335544592;
immutable int isc_dsql_max_arr_dim_exceeded        = 335544593;
immutable int isc_dsql_arr_range_error             = 335544594;
immutable int isc_dsql_trigger_err                 = 335544595;
immutable int isc_dsql_subselect_err               = 335544596;
immutable int isc_dsql_crdb_prepare_err            = 335544597;
immutable int isc_specify_field_err                = 335544598;
immutable int isc_num_field_err                    = 335544599;
immutable int isc_col_name_err                     = 335544600;
immutable int isc_where_err                        = 335544601;
immutable int isc_table_view_err                   = 335544602;
immutable int isc_distinct_err                     = 335544603;
immutable int isc_key_field_count_err              = 335544604;
immutable int isc_subquery_err                     = 335544605;
immutable int isc_expression_eval_err              = 335544606;
immutable int isc_node_err                         = 335544607;
immutable int isc_command_end_err                  = 335544608;
immutable int isc_index_name                       = 335544609;
immutable int isc_exception_name                   = 335544610;
immutable int isc_field_name                       = 335544611;
immutable int isc_token_err                        = 335544612;
immutable int isc_union_err                        = 335544613;
immutable int isc_dsql_construct_err               = 335544614;
immutable int isc_field_aggregate_err              = 335544615;
immutable int isc_field_ref_err                    = 335544616;
immutable int isc_order_by_err                     = 335544617;
immutable int isc_return_mode_err                  = 335544618;
immutable int isc_extern_func_err                  = 335544619;
immutable int isc_alias_conflict_err               = 335544620;
immutable int isc_procedure_conflict_error         = 335544621;
immutable int isc_relation_conflict_err            = 335544622;
immutable int isc_dsql_domain_err                  = 335544623;
immutable int isc_idx_seg_err                      = 335544624;
immutable int isc_node_name_err                    = 335544625;
immutable int isc_table_name                       = 335544626;
immutable int isc_proc_name                        = 335544627;
immutable int isc_idx_create_err                   = 335544628;
immutable int isc_wal_shadow_err                   = 335544629;
immutable int isc_dependency                       = 335544630;
immutable int isc_idx_key_err                      = 335544631;
immutable int isc_dsql_file_length_err             = 335544632;
immutable int isc_dsql_shadow_number_err           = 335544633;
immutable int isc_dsql_token_unk_err               = 335544634;
immutable int isc_dsql_no_relation_alias           = 335544635;
immutable int isc_indexname                        = 335544636;
immutable int isc_no_stream_plan                   = 335544637;
immutable int isc_stream_twice                     = 335544638;
immutable int isc_stream_not_found                 = 335544639;
immutable int isc_collation_requires_text          = 335544640;
immutable int isc_dsql_domain_not_found            = 335544641;
immutable int isc_index_unused                     = 335544642;
immutable int isc_dsql_self_join                   = 335544643;
immutable int isc_stream_bof                       = 335544644;
immutable int isc_stream_crack                     = 335544645;
immutable int isc_db_or_file_exists                = 335544646;
immutable int isc_invalid_operator                 = 335544647;
immutable int isc_conn_lost                        = 335544648;
immutable int isc_bad_checksum                     = 335544649;
immutable int isc_page_type_err                    = 335544650;
immutable int isc_ext_readonly_err                 = 335544651;
immutable int isc_sing_select_err                  = 335544652;
immutable int isc_psw_attach                       = 335544653;
immutable int isc_psw_start_trans                  = 335544654;
immutable int isc_invalid_direction                = 335544655;
immutable int isc_dsql_var_conflict                = 335544656;
immutable int isc_dsql_no_blob_array               = 335544657;
immutable int isc_dsql_base_table                  = 335544658;
immutable int isc_duplicate_base_table             = 335544659;
immutable int isc_view_alias                       = 335544660;
immutable int isc_index_root_page_full             = 335544661;
immutable int isc_dsql_blob_type_unknown           = 335544662;
immutable int isc_req_max_clones_exceeded          = 335544663;
immutable int isc_dsql_duplicate_spec              = 335544664;
immutable int isc_unique_key_violation             = 335544665;
immutable int isc_srvr_version_too_old             = 335544666;
immutable int isc_drdb_completed_with_errs         = 335544667;
immutable int isc_dsql_procedure_use_err           = 335544668;
immutable int isc_dsql_count_mismatch              = 335544669;
immutable int isc_blob_idx_err                     = 335544670;
immutable int isc_array_idx_err                    = 335544671;
immutable int isc_key_field_err                    = 335544672;
immutable int isc_no_delete                        = 335544673;
immutable int isc_del_last_field                   = 335544674;
immutable int isc_sort_err                         = 335544675;
immutable int isc_sort_mem_err                     = 335544676;
immutable int isc_version_err                      = 335544677;
immutable int isc_inval_key_posn                   = 335544678;
immutable int isc_no_segments_err                  = 335544679;
immutable int isc_crrp_data_err                    = 335544680;
immutable int isc_rec_size_err                     = 335544681;
immutable int isc_dsql_field_ref                   = 335544682;
immutable int isc_req_depth_exceeded               = 335544683;
immutable int isc_no_field_access                  = 335544684;
immutable int isc_no_dbkey                         = 335544685;
immutable int isc_jrn_format_err                   = 335544686;
immutable int isc_jrn_file_full                    = 335544687;
immutable int isc_dsql_open_cursor_request         = 335544688;
immutable int isc_ib_error                         = 335544689;
immutable int isc_cache_redef                      = 335544690;
immutable int isc_cache_too_small                  = 335544691;
immutable int isc_log_redef                        = 335544692;
immutable int isc_log_too_small                    = 335544693;
immutable int isc_partition_too_small              = 335544694;
immutable int isc_partition_not_supp               = 335544695;
immutable int isc_log_length_spec                  = 335544696;
immutable int isc_precision_err                    = 335544697;
immutable int isc_scale_nogt                       = 335544698;
immutable int isc_expec_short                      = 335544699;
immutable int isc_expec_long                       = 335544700;
immutable int isc_expec_ushort                     = 335544701;
immutable int isc_like_escape_invalid              = 335544702;
immutable int isc_svcnoexe                         = 335544703;
immutable int isc_net_lookup_err                   = 335544704;
immutable int isc_service_unknown                  = 335544705;
immutable int isc_host_unknown                     = 335544706;
immutable int isc_grant_nopriv_on_base             = 335544707;
immutable int isc_dyn_fld_ambiguous                = 335544708;
immutable int isc_dsql_agg_ref_err                 = 335544709;
immutable int isc_complex_view                     = 335544710;
immutable int isc_unprepared_stmt                  = 335544711;
immutable int isc_expec_positive                   = 335544712;
immutable int isc_dsql_sqlda_value_err             = 335544713;
immutable int isc_invalid_array_id                 = 335544714;
immutable int isc_extfile_uns_op                   = 335544715;
immutable int isc_svc_in_use                       = 335544716;
immutable int isc_err_stack_limit                  = 335544717;
immutable int isc_invalid_key                      = 335544718;
immutable int isc_net_init_error                   = 335544719;
immutable int isc_loadlib_failure                  = 335544720;
immutable int isc_network_error                    = 335544721;
immutable int isc_net_connect_err                  = 335544722;
immutable int isc_net_connect_listen_err           = 335544723;
immutable int isc_net_event_connect_err            = 335544724;
immutable int isc_net_event_listen_err             = 335544725;
immutable int isc_net_read_err                     = 335544726;
immutable int isc_net_write_err                    = 335544727;
immutable int isc_integ_index_deactivate           = 335544728;
immutable int isc_integ_deactivate_primary         = 335544729;
immutable int isc_cse_not_supported                = 335544730;
immutable int isc_tra_must_sweep                   = 335544731;
immutable int isc_unsupported_network_drive        = 335544732;
immutable int isc_io_create_err                    = 335544733;
immutable int isc_io_open_err                      = 335544734;
immutable int isc_io_close_err                     = 335544735;
immutable int isc_io_read_err                      = 335544736;
immutable int isc_io_write_err                     = 335544737;
immutable int isc_io_delete_err                    = 335544738;
immutable int isc_io_access_err                    = 335544739;
immutable int isc_udf_exception                    = 335544740;
immutable int isc_lost_db_connection               = 335544741;
immutable int isc_no_write_user_priv               = 335544742;
immutable int isc_token_too_long                   = 335544743;
immutable int isc_max_att_exceeded                 = 335544744;
immutable int isc_login_same_as_role_name          = 335544745;
immutable int isc_reftable_requires_pk             = 335544746;
immutable int isc_usrname_too_long                 = 335544747;
immutable int isc_password_too_long                = 335544748;
immutable int isc_usrname_required                 = 335544749;
immutable int isc_password_required                = 335544750;
immutable int isc_bad_protocol                     = 335544751;
immutable int isc_dup_usrname_found                = 335544752;
immutable int isc_usrname_not_found                = 335544753;
immutable int isc_error_adding_sec_record          = 335544754;
immutable int isc_error_modifying_sec_record       = 335544755;
immutable int isc_error_deleting_sec_record        = 335544756;
immutable int isc_error_updating_sec_db            = 335544757;
immutable int isc_sort_rec_size_err                = 335544758;
immutable int isc_bad_default_value                = 335544759;
immutable int isc_invalid_clause                   = 335544760;
immutable int isc_too_many_handles                 = 335544761;
immutable int isc_optimizer_blk_exc                = 335544762;
immutable int isc_invalid_string_constant          = 335544763;
immutable int isc_transitional_date                = 335544764;
immutable int isc_read_only_database               = 335544765;
immutable int isc_must_be_dialect_2_and_up         = 335544766;
immutable int isc_blob_filter_exception            = 335544767;
immutable int isc_exception_access_violation       = 335544768;
immutable int isc_exception_datatype_missalignment = 335544769;
immutable int isc_exception_array_bounds_exceeded  = 335544770;
immutable int isc_exception_float_denormal_operand = 335544771;
immutable int isc_exception_float_divide_by_zero   = 335544772;
immutable int isc_exception_float_inexact_result   = 335544773;
immutable int isc_exception_float_invalid_operand  = 335544774;
immutable int isc_exception_float_overflow         = 335544775;
immutable int isc_exception_float_stack_check      = 335544776;
immutable int isc_exception_float_underflow        = 335544777;
immutable int isc_exception_integer_divide_by_zero = 335544778;
immutable int isc_exception_integer_overflow       = 335544779;
immutable int isc_exception_unknown                = 335544780;
immutable int isc_exception_stack_overflow         = 335544781;
immutable int isc_exception_sigsegv                = 335544782;
immutable int isc_exception_sigill                 = 335544783;
immutable int isc_exception_sigbus                 = 335544784;
immutable int isc_exception_sigfpe                 = 335544785;
immutable int isc_ext_file_delete                  = 335544786;
immutable int isc_ext_file_modify                  = 335544787;
immutable int isc_adm_task_denied                  = 335544788;
immutable int isc_extract_input_mismatch           = 335544789;
immutable int isc_insufficient_svc_privileges      = 335544790;
immutable int isc_file_in_use                      = 335544791;
immutable int isc_service_att_err                  = 335544792;
immutable int isc_ddl_not_allowed_by_db_sql_dial   = 335544793;
immutable int isc_cancelled                        = 335544794;
immutable int isc_unexp_spb_form                   = 335544795;
immutable int isc_sql_dialect_datatype_unsupport   = 335544796;
immutable int isc_svcnouser                        = 335544797;
immutable int isc_depend_on_uncommitted_rel        = 335544798;
immutable int isc_svc_name_missing                 = 335544799;
immutable int isc_too_many_contexts                = 335544800;
immutable int isc_datype_notsup                    = 335544801;
immutable int isc_dialect_reset_warning            = 335544802;
immutable int isc_dialect_not_changed              = 335544803;
immutable int isc_database_create_failed           = 335544804;
immutable int isc_inv_dialect_specified            = 335544805;
immutable int isc_valid_db_dialects                = 335544806;
immutable int isc_sqlwarn                          = 335544807;
immutable int isc_dtype_renamed                    = 335544808;
immutable int isc_extern_func_dir_error            = 335544809;
immutable int isc_date_range_exceeded              = 335544810;
immutable int isc_inv_client_dialect_specified     = 335544811;
immutable int isc_valid_client_dialects            = 335544812;
immutable int isc_optimizer_between_err            = 335544813;
immutable int isc_service_not_supported            = 335544814;
immutable int isc_generator_name                   = 335544815;
immutable int isc_udf_name                         = 335544816;
immutable int isc_bad_limit_param                  = 335544817;
immutable int isc_bad_skip_param                   = 335544818;
immutable int isc_io_32bit_exceeded_err            = 335544819;
immutable int isc_invalid_savepoint                = 335544820;
immutable int isc_dsql_column_pos_err              = 335544821;
immutable int isc_dsql_agg_where_err               = 335544822;
immutable int isc_dsql_agg_group_err               = 335544823;
immutable int isc_dsql_agg_column_err              = 335544824;
immutable int isc_dsql_agg_having_err              = 335544825;
immutable int isc_dsql_agg_nested_err              = 335544826;
immutable int isc_exec_sql_invalid_arg             = 335544827;
immutable int isc_exec_sql_invalid_req             = 335544828;
immutable int isc_exec_sql_invalid_var             = 335544829;
immutable int isc_exec_sql_max_call_exceeded       = 335544830;
immutable int isc_conf_access_denied               = 335544831;
immutable int isc_wrong_backup_state               = 335544832;
immutable int isc_wal_backup_err                   = 335544833;
immutable int isc_cursor_not_open                  = 335544834;
immutable int isc_bad_shutdown_mode                = 335544835;
immutable int isc_concat_overflow                  = 335544836;
immutable int isc_bad_substring_offset             = 335544837;
immutable int isc_foreign_key_target_doesnt_exist  = 335544838;
immutable int isc_foreign_key_references_present   = 335544839;
immutable int isc_no_update                        = 335544840;
immutable int isc_cursor_already_open              = 335544841;
immutable int isc_stack_trace                      = 335544842;
immutable int isc_ctx_var_not_found                = 335544843;
immutable int isc_ctx_namespace_invalid            = 335544844;
immutable int isc_ctx_too_big                      = 335544845;
immutable int isc_ctx_bad_argument                 = 335544846;
immutable int isc_identifier_too_long              = 335544847;
immutable int isc_except2                          = 335544848;
immutable int isc_malformed_string                 = 335544849;
immutable int isc_prc_out_param_mismatch           = 335544850;
immutable int isc_command_end_err2                 = 335544851;
immutable int isc_partner_idx_incompat_type        = 335544852;
immutable int isc_bad_substring_length             = 335544853;
immutable int isc_charset_not_installed            = 335544854;
immutable int isc_collation_not_installed          = 335544855;
immutable int isc_att_shutdown                     = 335544856;
immutable int isc_blobtoobig                       = 335544857;
immutable int isc_must_have_phys_field             = 335544858;
immutable int isc_invalid_time_precision           = 335544859;
immutable int isc_blob_convert_error               = 335544860;
immutable int isc_array_convert_error              = 335544861;
immutable int isc_record_lock_not_supp             = 335544862;
immutable int isc_partner_idx_not_found            = 335544863;
immutable int isc_tra_num_exc                      = 335544864;
immutable int isc_field_disappeared                = 335544865;
immutable int isc_gfix_db_name                     = 335740929;
immutable int isc_gfix_invalid_sw                  = 335740930;
immutable int isc_gfix_incmp_sw                    = 335740932;
immutable int isc_gfix_replay_req                  = 335740933;
immutable int isc_gfix_pgbuf_req                   = 335740934;
immutable int isc_gfix_val_req                     = 335740935;
immutable int isc_gfix_pval_req                    = 335740936;
immutable int isc_gfix_trn_req                     = 335740937;
immutable int isc_gfix_full_req                    = 335740940;
immutable int isc_gfix_usrname_req                 = 335740941;
immutable int isc_gfix_pass_req                    = 335740942;
immutable int isc_gfix_subs_name                   = 335740943;
immutable int isc_gfix_wal_req                     = 335740944;
immutable int isc_gfix_sec_req                     = 335740945;
immutable int isc_gfix_nval_req                    = 335740946;
immutable int isc_gfix_type_shut                   = 335740947;
immutable int isc_gfix_retry                       = 335740948;
immutable int isc_gfix_retry_db                    = 335740951;
immutable int isc_gfix_exceed_max                  = 335740991;
immutable int isc_gfix_corrupt_pool                = 335740992;
immutable int isc_gfix_mem_exhausted               = 335740993;
immutable int isc_gfix_bad_pool                    = 335740994;
immutable int isc_gfix_trn_not_valid               = 335740995;
immutable int isc_gfix_unexp_eoi                   = 335741012;
immutable int isc_gfix_recon_fail                  = 335741018;
immutable int isc_gfix_trn_unknown                 = 335741036;
immutable int isc_gfix_mode_req                    = 335741038;
immutable int isc_gfix_opt_SQL_dialect             = 335741039;
immutable int isc_gfix_pzval_req                   = 335741042;
immutable int isc_dsql_dbkey_from_non_table        = 336003074;
immutable int isc_dsql_transitional_numeric        = 336003075;
immutable int isc_dsql_dialect_warning_expr        = 336003076;
immutable int isc_sql_db_dialect_dtype_unsupport   = 336003077;
immutable int isc_isc_sql_dialect_conflict_num     = 336003079;
immutable int isc_dsql_warning_number_ambiguous    = 336003080;
immutable int isc_dsql_warning_number_ambiguous1   = 336003081;
immutable int isc_dsql_warn_precision_ambiguous    = 336003082;
immutable int isc_dsql_warn_precision_ambiguous1   = 336003083;
immutable int isc_dsql_warn_precision_ambiguous2   = 336003084;
immutable int isc_dsql_ambiguous_field_name        = 336003085;
immutable int isc_dsql_udf_return_pos_err          = 336003086;
immutable int isc_dsql_invalid_label               = 336003087;
immutable int isc_dsql_datatypes_not_comparable    = 336003088;
immutable int isc_dsql_cursor_invalid              = 336003089;
immutable int isc_dsql_cursor_redefined            = 336003090;
immutable int isc_dsql_cursor_not_found            = 336003091;
immutable int isc_dsql_cursor_exists               = 336003092;
immutable int isc_dsql_cursor_rel_ambiguous        = 336003093;
immutable int isc_dsql_cursor_rel_not_found        = 336003094;
immutable int isc_dsql_cursor_not_open             = 336003095;
immutable int isc_dsql_type_not_supp_ext_tab       = 336003096;
immutable int isc_dyn_role_does_not_exist          = 336068796;
immutable int isc_dyn_no_grant_admin_opt           = 336068797;
immutable int isc_dyn_user_not_role_member         = 336068798;
immutable int isc_dyn_delete_role_failed           = 336068799;
immutable int isc_dyn_grant_role_to_user           = 336068800;
immutable int isc_dyn_inv_sql_role_name            = 336068801;
immutable int isc_dyn_dup_sql_role                 = 336068802;
immutable int isc_dyn_kywd_spec_for_role           = 336068803;
immutable int isc_dyn_roles_not_supported          = 336068804;
immutable int isc_dyn_domain_name_exists           = 336068812;
immutable int isc_dyn_field_name_exists            = 336068813;
immutable int isc_dyn_dependency_exists            = 336068814;
immutable int isc_dyn_dtype_invalid                = 336068815;
immutable int isc_dyn_char_fld_too_small           = 336068816;
immutable int isc_dyn_invalid_dtype_conversion     = 336068817;
immutable int isc_dyn_dtype_conv_invalid           = 336068818;
immutable int isc_dyn_zero_len_id                  = 336068820;
immutable int isc_gbak_unknown_switch              = 336330753;
immutable int isc_gbak_page_size_missing           = 336330754;
immutable int isc_gbak_page_size_toobig            = 336330755;
immutable int isc_gbak_redir_ouput_missing         = 336330756;
immutable int isc_gbak_switches_conflict           = 336330757;
immutable int isc_gbak_unknown_device              = 336330758;
immutable int isc_gbak_no_protection               = 336330759;
immutable int isc_gbak_page_size_not_allowed       = 336330760;
immutable int isc_gbak_multi_source_dest           = 336330761;
immutable int isc_gbak_filename_missing            = 336330762;
immutable int isc_gbak_dup_inout_names             = 336330763;
immutable int isc_gbak_inv_page_size               = 336330764;
immutable int isc_gbak_db_specified                = 336330765;
immutable int isc_gbak_db_exists                   = 336330766;
immutable int isc_gbak_unk_device                  = 336330767;
immutable int isc_gbak_blob_info_failed            = 336330772;
immutable int isc_gbak_unk_blob_item               = 336330773;
immutable int isc_gbak_get_seg_failed              = 336330774;
immutable int isc_gbak_close_blob_failed           = 336330775;
immutable int isc_gbak_open_blob_failed            = 336330776;
immutable int isc_gbak_put_blr_gen_id_failed       = 336330777;
immutable int isc_gbak_unk_type                    = 336330778;
immutable int isc_gbak_comp_req_failed             = 336330779;
immutable int isc_gbak_start_req_failed            = 336330780;
immutable int isc_gbak_rec_failed                  = 336330781;
immutable int isc_gbak_rel_req_failed              = 336330782;
immutable int isc_gbak_db_info_failed              = 336330783;
immutable int isc_gbak_no_db_desc                  = 336330784;
immutable int isc_gbak_db_create_failed            = 336330785;
immutable int isc_gbak_decomp_len_error            = 336330786;
immutable int isc_gbak_tbl_missing                 = 336330787;
immutable int isc_gbak_blob_col_missing            = 336330788;
immutable int isc_gbak_create_blob_failed          = 336330789;
immutable int isc_gbak_put_seg_failed              = 336330790;
immutable int isc_gbak_rec_len_exp                 = 336330791;
immutable int isc_gbak_inv_rec_len                 = 336330792;
immutable int isc_gbak_exp_data_type               = 336330793;
immutable int isc_gbak_gen_id_failed               = 336330794;
immutable int isc_gbak_unk_rec_type                = 336330795;
immutable int isc_gbak_inv_bkup_ver                = 336330796;
immutable int isc_gbak_missing_bkup_desc           = 336330797;
immutable int isc_gbak_string_trunc                = 336330798;
immutable int isc_gbak_cant_rest_record            = 336330799;
immutable int isc_gbak_send_failed                 = 336330800;
immutable int isc_gbak_no_tbl_name                 = 336330801;
immutable int isc_gbak_unexp_eof                   = 336330802;
immutable int isc_gbak_db_format_too_old           = 336330803;
immutable int isc_gbak_inv_array_dim               = 336330804;
immutable int isc_gbak_xdr_len_expected            = 336330807;
immutable int isc_gbak_open_bkup_error             = 336330817;
immutable int isc_gbak_open_error                  = 336330818;
immutable int isc_gbak_missing_block_fac           = 336330934;
immutable int isc_gbak_inv_block_fac               = 336330935;
immutable int isc_gbak_block_fac_specified         = 336330936;
immutable int isc_gbak_missing_username            = 336330940;
immutable int isc_gbak_missing_password            = 336330941;
immutable int isc_gbak_missing_skipped_bytes       = 336330952;
immutable int isc_gbak_inv_skipped_bytes           = 336330953;
immutable int isc_gbak_err_restore_charset         = 336330965;
immutable int isc_gbak_err_restore_collation       = 336330967;
immutable int isc_gbak_read_error                  = 336330972;
immutable int isc_gbak_write_error                 = 336330973;
immutable int isc_gbak_db_in_use                   = 336330985;
immutable int isc_gbak_sysmemex                    = 336330990;
immutable int isc_gbak_restore_role_failed         = 336331002;
immutable int isc_gbak_role_op_missing             = 336331005;
immutable int isc_gbak_page_buffers_missing        = 336331010;
immutable int isc_gbak_page_buffers_wrong_param    = 336331011;
immutable int isc_gbak_page_buffers_restore        = 336331012;
immutable int isc_gbak_inv_size                    = 336331014;
immutable int isc_gbak_file_outof_sequence         = 336331015;
immutable int isc_gbak_join_file_missing           = 336331016;
immutable int isc_gbak_stdin_not_supptd            = 336331017;
immutable int isc_gbak_stdout_not_supptd           = 336331018;
immutable int isc_gbak_bkup_corrupt                = 336331019;
immutable int isc_gbak_unk_db_file_spec            = 336331020;
immutable int isc_gbak_hdr_write_failed            = 336331021;
immutable int isc_gbak_disk_space_ex               = 336331022;
immutable int isc_gbak_size_lt_min                 = 336331023;
immutable int isc_gbak_svc_name_missing            = 336331025;
immutable int isc_gbak_not_ownr                    = 336331026;
immutable int isc_gbak_mode_req                    = 336331031;
immutable int isc_gbak_just_data                   = 336331033;
immutable int isc_gbak_data_only                   = 336331034;
immutable int isc_dsql_too_old_ods                 = 336397205;
immutable int isc_dsql_table_not_found             = 336397206;
immutable int isc_dsql_view_not_found              = 336397207;
immutable int isc_dsql_line_col_error              = 336397208;
immutable int isc_dsql_unknown_pos                 = 336397209;
immutable int isc_dsql_no_dup_name                 = 336397210;
immutable int isc_dsql_too_many_values             = 336397211;
immutable int isc_gsec_cant_open_db                = 336723983;
immutable int isc_gsec_switches_error              = 336723984;
immutable int isc_gsec_no_op_spec                  = 336723985;
immutable int isc_gsec_no_usr_name                 = 336723986;
immutable int isc_gsec_err_add                     = 336723987;
immutable int isc_gsec_err_modify                  = 336723988;
immutable int isc_gsec_err_find_mod                = 336723989;
immutable int isc_gsec_err_rec_not_found           = 336723990;
immutable int isc_gsec_err_delete                  = 336723991;
immutable int isc_gsec_err_find_del                = 336723992;
immutable int isc_gsec_err_find_disp               = 336723996;
immutable int isc_gsec_inv_param                   = 336723997;
immutable int isc_gsec_op_specified                = 336723998;
immutable int isc_gsec_pw_specified                = 336723999;
immutable int isc_gsec_uid_specified               = 336724000;
immutable int isc_gsec_gid_specified               = 336724001;
immutable int isc_gsec_proj_specified              = 336724002;
immutable int isc_gsec_org_specified               = 336724003;
immutable int isc_gsec_fname_specified             = 336724004;
immutable int isc_gsec_mname_specified             = 336724005;
immutable int isc_gsec_lname_specified             = 336724006;
immutable int isc_gsec_inv_switch                  = 336724008;
immutable int isc_gsec_amb_switch                  = 336724009;
immutable int isc_gsec_no_op_specified             = 336724010;
immutable int isc_gsec_params_not_allowed          = 336724011;
immutable int isc_gsec_incompat_switch             = 336724012;
immutable int isc_gsec_inv_username                = 336724044;
immutable int isc_gsec_inv_pw_length               = 336724045;
immutable int isc_gsec_db_specified                = 336724046;
immutable int isc_gsec_db_admin_specified          = 336724047;
immutable int isc_gsec_db_admin_pw_specified       = 336724048;
immutable int isc_gsec_sql_role_specified          = 336724049;
immutable int isc_license_no_file                  = 336789504;
immutable int isc_license_op_specified             = 336789523;
immutable int isc_license_op_missing               = 336789524;
immutable int isc_license_inv_switch               = 336789525;
immutable int isc_license_inv_switch_combo         = 336789526;
immutable int isc_license_inv_op_combo             = 336789527;
immutable int isc_license_amb_switch               = 336789528;
immutable int isc_license_inv_parameter            = 336789529;
immutable int isc_license_param_specified          = 336789530;
immutable int isc_license_param_req                = 336789531;
immutable int isc_license_syntx_error              = 336789532;
immutable int isc_license_dup_id                   = 336789534;
immutable int isc_license_inv_id_key               = 336789535;
immutable int isc_license_err_remove               = 336789536;
immutable int isc_license_err_update               = 336789537;
immutable int isc_license_err_convert              = 336789538;
immutable int isc_license_err_unk                  = 336789539;
immutable int isc_license_svc_err_add              = 336789540;
immutable int isc_license_svc_err_remove           = 336789541;
immutable int isc_license_eval_exists              = 336789563;
immutable int isc_gstat_unknown_switch             = 336920577;
immutable int isc_gstat_retry                      = 336920578;
immutable int isc_gstat_wrong_ods                  = 336920579;
immutable int isc_gstat_unexpected_eof             = 336920580;
immutable int isc_gstat_open_err                   = 336920605;
immutable int isc_gstat_read_err                   = 336920606;
immutable int isc_gstat_sysmemex                   = 336920607;
immutable int isc_err_max                          = 763;
