module jp.ku6.bbs.dao.ThreadDao;

import std.datetime;
import jp.ku6.firebird.core;
import jp.ku6.bbs.entity.ThreadEntity;
import jp.ku6.bbs.model.Thread;

/++
 + スレッドDAO
 +/
class ThreadDao
{
	/++
	 + コンストラクタ
	 +
	 + Params:
	 +  transaction = SQL実行に使用するトランザクションオブジェクト
	 +/
	this(Transaction transaction)
	{
		selectNextSequence_ = new Statement("SELECT GEN_ID(thread_sequence, 1) FROM RDB$DATABASE", transaction);
		selectForListView_ = new Statement("SELECT t.id, t.title, t.registered, COUNT(t.id), MAX(r.registered) FROM responses AS r LEFT JOIN threads AS t ON r.thread_id = t.id GROUP BY t.id, t.title, t.registered", transaction);
		insert_ = new Statement("INSERT INTO threads(id, title, registered) VALUES(?, ?, ?)", transaction);
	}


	/++
	 + 次のーケンス番号を取得
	 +
	 + Returns:
	 +  次のシーケンス番号
	 +/
	uint selectNextSequence()
	{
		Statement statement = selectNextSequence_;

		statement.prepare();
		scope(exit) statement.finish();

		auto result = statement.execute();
		return result.fetch() ? result.get!(uint)(0) : 0;
	}

	/++
	 + キーワード毎の記事件数を取得
	 +
	 + Returns:
	 +  キーワードIDがキー、記事件数が値の連想配列
	 +/
	Thread[] selectForListView()
	{
		Statement statement = selectForListView_;

		// SQL実行の準備と終了処理
		statement.prepare();
		scope(exit) statement.finish();

		// SQL実行
		auto result = statement.execute();

		// データ取得
		Thread[] threads;
		while (result.fetch())
		{
			auto thread = new Thread;

			// データベースから取得したデータを型指定して取り出す
			thread.id = result.get!(uint)(0);
			thread.title = result.get!(string)(1);
			thread.registered = result.get!(SysTime)(2);
			thread.responseCount = result.get!(uint)(3);
			thread.updated = result.get!(SysTime)(4);

			threads ~= thread;
		}

		return threads;
	}

	/++
	 + 登録
	 +
	 + Params:
	 +  report = 登録データ
	 +/
	void insert(ThreadEntity entity)
	{
		Statement statement = insert_;

		statement.prepare();
		scope(exit) statement.finish();

		statement.set(0u, entity.id);
		statement.set(1u, entity.title);
		statement.set(2u, entity.registered);
		statement.execute();
	}

private:

	// 各SQLオブジェクト
	Statement selectNextSequence_;
	Statement selectForListView_;
	Statement insert_;
}
