module jp.ku6.bbs.main;

import jp.ku6.raptan.core;
import jp.ku6.raptan.filter;
import jp.ku6.firebird.extension;
import jp.ku6.bbs.constants;
import jp.ku6.bbs.action.ThreadListAction;
import jp.ku6.bbs.action.ThreadBuildAction;
import jp.ku6.bbs.logic.ThreadLogic;

void main(string[] arguments)
{
	// コネクションプールの作成
	ConnectionPool connectionPool = new ConnectionPool(DATABASE_PATH, DATABASE_USER, DATABASE_PASS, 10);
	scope(exit) connectionPool.shutdown();

	// スレッドロジックの作成
	auto threadLogic = new ThreadLogic(connectionPool);

	// フレームワークオブジェクトの生成
	auto raptan = new Raptan(2 * 1024 * 1024, 0, "/tmp");

	// 拡張子を判断して自動でContent-Typeを出力する設定
	auto contentTypeFilter = new ContentTypeFilter;
	contentTypeFilter.add("html", "text/html; charset=UTF-8");
	raptan.addFilter(contentTypeFilter);

	// リクエストメソッドフィルタリング設定
	raptan.addFilter(new RequestMethodFilter(Request.Method.GET | Request.Method.POST));

	// パスに対して処理を設定
	auto threadListAction = new ThreadListAction(threadLogic);
	raptan.setAction(APPLICATION_ROOT_URI ~ "/ThreadList.html", &threadListAction.execute);

	auto threadBuildAction = new ThreadBuildAction(threadLogic);
	raptan.setAction(APPLICATION_ROOT_URI ~ "/ThreadBuild.html", &threadBuildAction.execute);

	// リクエストの受付を開始
	raptan.start();

	// アプリケーションの終了を待機
	raptan.join();
}
