module jp.ku6.bbs.action.BaseView;

import jp.ku6.sweets.core;
import jp.ku6.sweets.extension;
import jp.ku6.bbs.constants;

/++
 + 共通表示処理
 +/
class BaseView
{
	/++
	 + コンストラクタ
	 +
	 + Params:
	 +  filePath = テンプレートディレクトリ以下のパスを書く("/"から開始)
	 +/
	this(string filePath)
	{
		// HTMLファイルからテンプレートオブジェクトを作成
		auto tpl = FileAutoReloadableSweetsTemplate.instance(TEMPLATE_DIRECTORY ~ filePath);

		// テンプレートオブジェクトをテンプレート処理オブジェクトに渡す
		contents_ = new AutoUpdatableSweets(tpl);

		// s:id="link"が付いたタグに対する処理を割り当て
		contents_.assign("link", &linkProcessor);
	}

	/++
	 + 表示処理
	 +
	 + Params:
	 +  response = レスポンスオブジェクト
	 +/
	void print(SweetsPrinter printer)
	{
		contents_.print(printer);
	}

protected:

	// テンプレート処理オブジェクト
	AutoUpdatableSweets contents_;

	// リンクにアプリのパスを追加
	void linkProcessor(SweetsPrinter printer, SweetsElement element, const(SweetsNode[]) children)
	{
		// s:attributeで指定された属性名を取得する
		const(string)* target = "s:attribute" in element.attributes;
		if (target !is null)
		{
			// 指定された属性名の値を取得
			const(string)* v = *target in element.attributes;
			if (v !is null)
			{
				// 指定された属性名の値に、アプリケーションのURLのrootを付ける
				string[string] attributes;
				attributes[*target] = APPLICATION_ROOT_URI ~ *v;

				// 子要素が無いタグならば単一タグとして出力 (<homu/>)
				if (children is null || children.length == 0)
					element.printStartTagWithAdditionalAttributes(printer, attributes, true);
				// 子要素があるタグならば通常のタグとして出力 (<homu>～</homu>)
				else
				{
					element.printStartTagWithAdditionalAttributes(printer, attributes, false);
					foreach (child; children)
						child.print(printer);
					element.printEndTag(printer);
				}

				return;
			}
		}

		// 期待した記述ではないのでとりあえず普通にタグを出力する
		element.printStartTag(printer, false);
		foreach (child; children)
			child.print(printer);
		element.printEndTag(printer);
	}
}
